package batePapo.jadix.controle;

import java.util.List;

import batePapo.jadix.controle.Acao;
import edugraf.jadix.ApliqueSemPichador;
import edugraf.jadix.eventos.EventoDeTeclado;
import edugraf.jadix.eventos.EventoSimples;
import edugraf.jadix.eventos.nomes.NomeDeEventosDeTeclado;
import edugraf.jadix.eventos.nomes.NomeDeEventosSimples;
import edugraf.jadix.fachada.ComponenteDix;
import edugraf.jadix.fachada.PaginaDix;
import edugraf.jadix.fachada.TratadorDeEventos;
import edugraf.jadix.fachada.TratadorDixAbstrato;
import edugraf.jadix.tiposPrimitivos.Palavra;

public class Controle extends ApliqueSemPichador {
	private PaginaDix página;
	private ComponenteDix campoDeDigitaçãoDeMensagem;
	private ComponenteDix áreaDeMensagens;
	private ComponenteDix botãoEntrar;
	private ComponenteDix botãoEnviarMensagem;
	private ComponenteDix campoApelido;
	private ComponenteDix etiquetaTítulo;
	private ComponenteDix etiquetaInformativa;
	
	@Override
	public void iniciar() {
		página = this.obterPaginaDix();
		campoDeDigitaçãoDeMensagem = this.página.criarProcuradorDeComponente("campoDeDigitaçãoDeMensagem");
		áreaDeMensagens = this.página.criarProcuradorDeComponente("áreaDeMensagens");
		botãoEntrar = this.página.criarProcuradorDeComponente("botãoEntrar");
		campoApelido = this.página.criarProcuradorDeComponente("campoApelido");
		etiquetaInformativa = this.página.criarProcuradorDeComponente("etiquetaInformativa");
		etiquetaTítulo = this.página.criarProcuradorDeComponente("etiquetaTítulo");
		botãoEnviarMensagem = this.página.criarProcuradorDeComponente("botãoEnviarMensagem");
		instalarTratadoresDeEventos();
		instalarTratadoresDeEstimulos();
	}
	
	private void instalarTratadoresDeEventos() {
		botãoEntrar.adicionarTratadorDeEventos(new TratadorDeCliques(new AçãoEntrarNoBatePapo()));
		botãoEnviarMensagem.adicionarTratadorDeEventos(new TratadorDeCliques(new AçãoEnviarMensagem()));
		campoApelido.adicionarTratadorDeEventos(new TratadorDeTeclaEnterEmCampo(new AçãoEntrarNoBatePapo()));
		campoDeDigitaçãoDeMensagem.adicionarTratadorDeEventos(new TratadorDeTeclaEnterEmCampo(new AçãoEnviarMensagem()));
	}

	private void instalarTratadoresDeEstimulos() {
		página.instalarTratador("\"controle\" \"batePapo\" \"mensagemRecebida\" texto", new TratadorMostrarMensagens());
	}


	private void mostrarMensagem(String mensagem) {
		áreaDeMensagens.fixarTexto(áreaDeMensagens.obterTexto() + "\n" + mensagem);
	}

	private void executarAjustesPósEnvioDeMensagem() {
		campoDeDigitaçãoDeMensagem.fixarTexto("");
		campoDeDigitaçãoDeMensagem.fixarFoco();
	}

	private class TratadorMostrarMensagens implements TratadorDeEventos {

		public void seDito(List<Palavra> evento) {
			mostrarMensagem(evento.get(3).comoTexto());			
		}
		
	}
	
	private class TratadorDeTeclaEnterEmCampo extends TratadorDixAbstrato {
		private final Acao _ação;

		public TratadorDeTeclaEnterEmCampo(Acao ação) {
			_ação = ação;
		}

		public void seDito(EventoDeTeclado evento) {
			if (evento.obterTecla() == 13 && evento.obterNomeDoEvento().equals(NomeDeEventosDeTeclado.TECLA_SOLTA))
				_ação.executar();
		}
	}

	private class TratadorDeCliques extends TratadorDixAbstrato {
		private final Acao _ação;

		public TratadorDeCliques(Acao ação) {
			_ação = ação;
		}

		@Override
		public void seDito(EventoSimples evento) {
			if (evento.obterNomeDoEvento().equals(NomeDeEventosSimples.CLICADO))
				_ação.executar();
		}
	}
	
	class AçãoEntrarNoBatePapo implements Acao {

		public void executar() {
			notificarModeloParaEntrarNoBatePapo(campoApelido.obterTexto());						
			etiquetaTítulo.fixarTexto(campoApelido.obterTexto());
			configurarInterfaceParaEnvioDeMensagens();
		}

	}

	private void notificarModeloParaEntrarNoBatePapo(String apelido) {
		página.fixarParametro(apelido).dizer("\"modelo\" \"batePapo\" \"entrarNoBatePapo\" texto");
	}
	
	private void configurarInterfaceParaEnvioDeMensagens() {
		botãoEntrar.desabilitar();
		botãoEntrar.tornarInvisivel();
		campoApelido.desabilitar();
		campoApelido.tornarInvisivel();
		etiquetaInformativa.tornarInvisivel();
		campoDeDigitaçãoDeMensagem.tornarVisivel();
		botãoEnviarMensagem.tornarVisivel();
		áreaDeMensagens.tornarVisivel();
	}

	class AçãoEnviarMensagem implements Acao {
		public void executar() {
			notificarModeloParaEnviarMensagem(campoDeDigitaçãoDeMensagem.obterTexto());
			executarAjustesPósEnvioDeMensagem();
		}
	}
	
	private void notificarModeloParaEnviarMensagem(String mensagem) {
		página.fixarParametro(mensagem).dizer("\"modelo\" \"batePapo\" \"enviarMensagem\" texto");
	}

}
