package mostrarTextos;

import java.awt.Rectangle;

import edugraf.pichador.TipoPichador;
import edugraf.pichador.desenho.Cor;

public class FolhaDeCaderno {

	private final TipoPichador pichador;
	private final int espaçamento;
	private final int margemEsquerda;
	private final Rectangle áreaDaFolha;

	public FolhaDeCaderno(TipoPichador pichador, Rectangle áreaDaFolha, int espaçamento) {
		this.pichador = pichador;
		this.áreaDaFolha = áreaDaFolha;
		this.espaçamento = espaçamento;
		this.margemEsquerda = (int) áreaDaFolha.getMinX() + 20;
		desenharLinhasDoCaderno();
		posicionarCanetaNoInício(pichador, áreaDaFolha);
	}

	private void posicionarCanetaNoInício(TipoPichador pichador, Rectangle áreaDaFolha) {
		int inícioHorizontalDaEscrita = margemEsquerda + pichador.obterEspessura();
		pichador
			.semRastros()
			.andarPara(inícioHorizontalDaEscrita, (int) áreaDaFolha.getMaxY())
			.fixarDireção(270);
	}
	
	public void mostrarEtiquetaEmNovaLinha(String texto) {
		pichador.frente(espaçamento).mostrarEtiqueta(texto);
	}

	public void mostrarEmNovaLinha(String texto) {
		pichador.frente(espaçamento).mostrar(texto);
	}	

	private void desenharLinhasDoCaderno() {
		pichador.fixarCorDosRastros(new Cor(192, 192, 192));
		desenharLinhasHorizontais(espaçamento);
		
		pichador.fixarCorDosRastros(new Cor(255, 0, 0));
		desenharLinhaVertical(margemEsquerda);
	}

	private void desenharLinhaVertical(int x) {
		pichador
			.semRastros()
			.andarPara(x, (int) áreaDaFolha.getMinY())
			.comRastros()
			.andarPara(x, (int) áreaDaFolha.getMaxY());
	}

	private void desenharLinhasHorizontais(int espaçamento) {
		for (int y = (int) áreaDaFolha.getMinX(); y <= áreaDaFolha.getMaxX(); y += espaçamento)
			desenharLinhaHorizontal(y);
	}

	private void desenharLinhaHorizontal(int y) {
		pichador
			.semRastros()
			.andarPara((int) áreaDaFolha.getMinX(), y)
			.comRastros()
			.andarPara((int) áreaDaFolha.getMaxX(), y);
	}	

}
