package componentes.abstratos.gedix;

import static infraestrutura.AtributosDosComponentesGedix.altura;
import static infraestrutura.AtributosDosComponentesGedix.esquerda;
import static infraestrutura.AtributosDosComponentesGedix.habilitado;
import static infraestrutura.AtributosDosComponentesGedix.largura;
import static infraestrutura.AtributosDosComponentesGedix.nome;
import static infraestrutura.AtributosDosComponentesGedix.posioDaLegenda;
import static infraestrutura.AtributosDosComponentesGedix.profundidade;
import static infraestrutura.AtributosDosComponentesGedix.textoDaLegenda;
import static infraestrutura.AtributosDosComponentesGedix.tipoDoComponente;
import static infraestrutura.AtributosDosComponentesGedix.topo;
import static infraestrutura.AtributosDosComponentesGedix.visvel;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.PosicoesDeLegenda;
import infraestrutura.TiposDeComponentesGedix;
import java.util.HashMap;
import java.util.Map;


public abstract class ComponenteGedix implements TipoComponenteGedix {

	protected Map<AtributosDosComponentesGedix, String> mapeamentoAtributoValor = new HashMap<AtributosDosComponentesGedix, String>();

	public ComponenteGedix() {
		fixarLargura(5);
		fixarAltura(5);
		fixarEsquerda(0);
		fixarTopo(0);
		fixarProfundidade(0);
		tornarVisvel();
		habilitar();
		fixarLegenda("");
		fixarPosioDeLegenda(PosicoesDeLegenda.emCima);
	}

	public void desabilitar() {
		mapeamentoAtributoValor.put(habilitado, "false");
	}

	public boolean equals(Object outro) {
		if (outro instanceof TipoComponenteGedix)
			return ((TipoComponenteGedix) outro).obterMapeamentoAtributoValor().equals(
					obterMapeamentoAtributoValor());

		return false;
	}

	public boolean estHabilitado() {
		return Boolean.parseBoolean(mapeamentoAtributoValor.get(habilitado));
	}

	public boolean estVisvel() {
		return Boolean.parseBoolean(mapeamentoAtributoValor.get(visvel));
	}

	public void fixarAltura(int novaAltura) {
		assert novaAltura > 0;
		mapeamentoAtributoValor.put(altura, String.valueOf(novaAltura));
	}

	public void fixarEsquerda(int novaEsquerda) {
		assert novaEsquerda >= 0;
		mapeamentoAtributoValor.put(esquerda, String.valueOf(novaEsquerda));
	}

	public void fixarLargura(int novaLargura) {
		assert novaLargura > 0;
		mapeamentoAtributoValor.put(largura, String.valueOf(novaLargura));
	}

	public void fixarLegenda(String novaLegenda) {
		mapeamentoAtributoValor.put(textoDaLegenda, novaLegenda);
	}

	public void fixarPosioDeLegenda(PosicoesDeLegenda novaPosio) {
		mapeamentoAtributoValor.put(posioDaLegenda, novaPosio.toString());
	}

	public void fixarProfundidade(int novaProfundidade) {
		mapeamentoAtributoValor.put(profundidade, String.valueOf(novaProfundidade));
	}

	public void fixarTopo(int novoTopo) {
		assert novoTopo >= 0;
		mapeamentoAtributoValor.put(topo, String.valueOf(novoTopo));
	}

	public String gerarFimDoXML() {
		return "</" + obterTipo() + ">";
	}

	protected String gerarParteBsicaDoXML() {
		final String xml = "<" + obterTipo() + " nome=\"" + obterNome() + "\" topo=\""
				+ obterTopo() + "\" esquerda=\"" + obterEsquerda() + "\" altura=\""
				+ obterAltura() + "\" largura=\"" + obterLargura() + "\" legenda=\""
				+ obterLegenda() + "\" posioDaLegenda=\"" + obterPosioDeLegenda()
				+ "\" profundidade=\"" + obterProfundidade() + "\" habilitado=\""
				+ estHabilitado() + "\" visvel=\"" + estVisvel() + "\"";
		return xml;
	}

	public String gerarXML() {
		return gerarParteBsicaDoXML() + gerarFimDoXML();
	}

	public void habilitar() {
		mapeamentoAtributoValor.put(habilitado, "true");
	}

	public int hashCode() {
		return mapeamentoAtributoValor.hashCode();
	}

	public int obterAltura() {
		return Integer.parseInt(mapeamentoAtributoValor.get(altura));
	}

	public int obterEsquerda() {
		return Integer.parseInt(mapeamentoAtributoValor.get(esquerda));
	}

	public int obterLargura() {
		return Integer.parseInt(mapeamentoAtributoValor.get(largura));
	}

	public String obterLegenda() {
		return mapeamentoAtributoValor.get(textoDaLegenda);
	}

	public Map<AtributosDosComponentesGedix, String> obterMapeamentoAtributoValor() {
		final Map<AtributosDosComponentesGedix, String> mapaARetornar = new HashMap<AtributosDosComponentesGedix, String>();
		mapaARetornar.putAll(mapeamentoAtributoValor);

		return mapaARetornar;
	}

	public String obterNome() {
		return mapeamentoAtributoValor.get(nome);
	}

	public PosicoesDeLegenda obterPosioDeLegenda() {
		return PosicoesDeLegenda.valueOf(mapeamentoAtributoValor.get(posioDaLegenda));
	}

	public int obterProfundidade() {
		return Integer.parseInt(mapeamentoAtributoValor.get(profundidade));
	}

	public TiposDeComponentesGedix obterTipo() {
		return TiposDeComponentesGedix.valueOf(mapeamentoAtributoValor
				.get(tipoDoComponente));
	}

	public int obterTopo() {
		return Integer.parseInt(mapeamentoAtributoValor.get(topo));
	}

	public void tornarInvisvel() {
		mapeamentoAtributoValor.put(visvel, "false");
	}

	public void tornarVisvel() {
		mapeamentoAtributoValor.put(visvel, "true");
	}

	public String toString() {
		return mapeamentoAtributoValor.toString();
	}

}