package controle.barraDeFerramentas;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import controle.barraDeFerramentas.tratadoresDix.TratadorDeMudarDeAba;
import edugraf.jadix.fachada.ComponenteDix;
import edugraf.jadix.fachada.NotificavelPorDix;
import edugraf.jadix.fachada.PaginaDix;


public class BarraDeFerramentas implements TipoBarraDeFerramentas {

	private final String[] componentesCriao = { "boto", "grade", "imagem", "etiqueta",
			"campoDeTexto", "reaDeTexto", "caixaDeEscolha", "listaDeEscolha",
			"botesDeRdio", "reaSensvel" };

	private final Set<ComponenteDix> procsDeCriao = new HashSet<ComponenteDix>();
	private final Set<ComponenteDix> procsDeEdio = new HashSet<ComponenteDix>();

	private ComponenteDix endereo, log;
	private ComponenteDix botoCriao, botoEdio, botoSalvar, botoAbrir, botoXML;
	private ComponenteDix listaDeComponentes, remover;

	private final PaginaDix pgina;

	public BarraDeFerramentas(PaginaDix pginaDaBarra) {
		pgina = pginaDaBarra;
		criarProcuradores();

		final NotificavelPorDix tratadorDeAba = new TratadorDeMudarDeAba(this);

		botoCriao.adicionarTratadorDeEventos(tratadorDeAba);
		botoEdio.adicionarTratadorDeEventos(tratadorDeAba);
	}

	public void adicionarNaListaDeComponentes(String nome) {
		final List<String> atual = listaDeComponentes.obterLista();
		atual.add(nome);
		listaDeComponentes.fixarLista(atual);
	}

	public void colocarNoLog(String mensagem) {
		log.fixarTexto(mensagem);
	}

	private void criarProcuradores() {
		for (final String nome : componentesCriao)
			procsDeCriao.add(pgina.criarProcuradorDeComponente(nome));

		endereo = pgina.criarProcuradorDeComponente("endereco");
		botoCriao = pgina.criarProcuradorDeComponente("criao");
		botoEdio = pgina.criarProcuradorDeComponente("edio");
		botoAbrir = pgina.criarProcuradorDeComponente("abrir");
		botoSalvar = pgina.criarProcuradorDeComponente("salvar");
		botoXML = pgina.criarProcuradorDeComponente("xml");
		listaDeComponentes = pgina.criarProcuradorDeComponente("listaDeComponentes");
		log = pgina.criarProcuradorDeComponente("log");
		remover = pgina.criarProcuradorDeComponente("remover");

		procsDeEdio.add(listaDeComponentes);
		procsDeEdio.add(remover);
	}

	public void instalarTratadorNaListaDeComponentes(NotificavelPorDix tratador) {
		listaDeComponentes.adicionarTratadorDeEventos(tratador);
	}

	public void instalarTratadorNoBotaoAbrir(NotificavelPorDix tratador) {
		botoAbrir.adicionarTratadorDeEventos(tratador);
	}

	public void instalarTratadorNoBotoRemover(NotificavelPorDix tratador) {
		remover.adicionarTratadorDeEventos(tratador);
	}

	public void instalarTratadorNoBotaoSalvar(NotificavelPorDix tratador) {
		botoSalvar.adicionarTratadorDeEventos(tratador);
	}

	public void instalarTratadorNoBotaoXML(NotificavelPorDix tratador) {
		botoXML.adicionarTratadorDeEventos(tratador);
	}

	public void instalarTratadorNosBotesDeCriao(NotificavelPorDix tratador) {
		for (final ComponenteDix boto : procsDeCriao)
			boto.adicionarTratadorDeEventos(tratador);
	}

	public void limparLista() {
		listaDeComponentes.fixarLista(new ArrayList<String>());
	}

	public void mudarParaAbaCriao() {
		for (final ComponenteDix c : procsDeCriao)
			c.tornarVisivel();

		for (final ComponenteDix c : procsDeEdio)
			c.tornarInvisivel();
	}

	public void mudarParaAbaEdio() {
		for (final ComponenteDix c : procsDeEdio)
			c.tornarVisivel();

		for (final ComponenteDix c : procsDeCriao)
			c.tornarInvisivel();
	}

	public ComponenteDix obterBotoDaAbaCriao() {
		return botoCriao;
	}

	public ComponenteDix obterBotoDaAbaEdio() {
		return botoEdio;
	}

	public String obterEndereo() {
		return endereo.obterTexto();
	}

	public void removerDaLista(String nomeDoComponente) {
		final List<String> opes = listaDeComponentes.obterLista();
		opes.remove(nomeDoComponente);
		listaDeComponentes.fixarLista(opes);
	}

}
