package controle.barraDeFerramentas;

import controle.barraDeFerramentas.tratadoresDeRede.TratadorDeAdicionarNaLista;
import controle.barraDeFerramentas.tratadoresDeRede.TratadorDeColocarNoLog;
import controle.barraDeFerramentas.tratadoresDeRede.TratadorDeLimparLista;
import controle.barraDeFerramentas.tratadoresDeRede.TratadorDeRemoverDaLista;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirParaAbrir;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirRemocao;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirSelecao;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirSerializacao;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirTrocaDeTipo;
import controle.barraDeFerramentas.tratadoresDix.TratadorDePedirXML;
import edugraf.jadix.ApliqueSemPichador;
import edugraf.jadix.fachada.GerenteDeRede;
import edugraf.jadix.fachada.PaginaDix;


public class ControladorDaBarra extends ApliqueSemPichador implements
		TipoControladorDaBarra {

	private PaginaDix pgina;
	private TipoBarraDeFerramentas barra;
	private GerenteDeRede gerenteDeRede;

	public void adicionarComponenteNaLista(String nomeDoComponente) {
		barra.adicionarNaListaDeComponentes(nomeDoComponente);
	}

	public void iniciar() {
		pgina = obterPaginaDix();
		gerenteDeRede = obterGerenteDeRede();
		barra = new BarraDeFerramentas(pgina);

		instalarTratadores();
	}

	private void instalarTratadores() {
		barra.instalarTratadorNosBotesDeCriao(new TratadorDePedirTrocaDeTipo(this));
		barra.instalarTratadorNaListaDeComponentes(new TratadorDePedirSelecao(this));
		barra.instalarTratadorNoBotoRemover(new TratadorDePedirRemocao(this));
		barra.instalarTratadorNoBotaoSalvar(new TratadorDePedirSerializacao(this));
		barra.instalarTratadorNoBotaoAbrir(new TratadorDePedirParaAbrir(this));
		barra.instalarTratadorNoBotaoXML(new TratadorDePedirXML(this));

		gerenteDeRede.instalarTratador("\"colocarNoLog\" texto",
				new TratadorDeColocarNoLog(barra));
		gerenteDeRede.instalarTratador("\"adicionarNaListaDeComponentes\" texto",
				new TratadorDeAdicionarNaLista(this));
		gerenteDeRede.instalarTratador("\"removerDaListaDeComponentes\" texto",
				new TratadorDeRemoverDaLista(this));
		gerenteDeRede.instalarTratador("\"limparListaDeComponentes\"",
				new TratadorDeLimparLista(this));
	}

	public void limparListaDeComponentes() {
		barra.limparLista();
	}

	public void pedirParaAbrir() {
		final String endereo = barra.obterEndereo();
		gerenteDeRede.fixarParametro(endereo).dizer("\"abrirArquivo\" texto");
	}

	public void pedirRemooDeComponente() {
		gerenteDeRede.dizer("\"removerComponenteSelecionado\"");
	}

	public void pedirSerializao() {
		final String endereo = barra.obterEndereo();
		gerenteDeRede.fixarParametro(endereo).dizer("\"salvarArquivo\" texto");
	}

	public void pedirTrocaDeComponenteAEditar(String nomeDoComponenteAEditar) {
		gerenteDeRede.fixarParametro(nomeDoComponenteAEditar).dizer(
				"\"trocarComponenteDeEdicao\" texto");
	}

	public void pedirTrocaDeTipoACriar(String nomeDoTipoSelecionado) {
		gerenteDeRede.fixarParametro(nomeDoTipoSelecionado).dizer(
				"\"trocarTipoDeCriacao\" texto");
	}

	public void pedirXML() {
		final String endereo = barra.obterEndereo();
		gerenteDeRede.fixarParametro(endereo).dizer("\"gerarXML\" texto");
	}

	public void removerDaListaDeComponentes(String nomeDoComponente) {
		barra.removerDaLista(nomeDoComponente);
	}

}