package controle.janelaDeEdicao;

import java.util.HashSet;
import java.util.Set;
import edugraf.jadix.fachada.ComponenteDix;
import edugraf.jadix.fachada.PaginaDix;
import edugraf.jadix.fachada.TratadorDixAbstrato;


public class JanelaDeEdicao implements TipoJanelaDeEdicao {

	private final String[] camposDeComponentes = { "topo", "esquerda", "altura",
			"largura", "profundidade", "visvel", "habilitado", "textoDaLegenda",
			"posioDaLegenda" };

	private final String[] camposDeGrade = { "linhas", "colunas" };

	private final String[] camposDeComponenteComOpcao = { "opes", "remover_opo",
			"adicionar_opo", "nova_opo" };

	private final String campoTexto = "texto";

	private final String campoNome = "nome";

	private final String campoUri = "uri";

	private final Set<ComponenteDix> procsDeCampos = new HashSet<ComponenteDix>();
	private final Set<ComponenteDix> procsDeCamposGrade = new HashSet<ComponenteDix>();
	private final Set<ComponenteDix> procsDeCamposOpo = new HashSet<ComponenteDix>();
	private ComponenteDix procDeTexto;
	private ComponenteDix procDeURI;

	private final PaginaDix pgina;

	private ComponenteDix procDeNome;

	public JanelaDeEdicao(PaginaDix pgina) {
		this.pgina = pgina;

		criarProcuradores();
	}

	private void criarProcuradores() {

		for (final String nome : camposDeComponentes)
			procsDeCampos.add(pgina.criarProcuradorDeComponente(nome));

		for (final String nome : camposDeComponenteComOpcao)
			procsDeCamposOpo.add(pgina.criarProcuradorDeComponente(nome));

		for (final String nome : camposDeGrade)
			procsDeCamposGrade.add(pgina.criarProcuradorDeComponente(nome));

		procDeTexto = pgina.criarProcuradorDeComponente(campoTexto);
		procDeURI = pgina.criarProcuradorDeComponente(campoUri);
		procDeNome = pgina.criarProcuradorDeComponente(campoNome);
	}

	public void instalarTratadorNosCampos(TratadorDixAbstrato tratador) {

		final Set<ComponenteDix> todosOsProcs = new HashSet<ComponenteDix>();
		todosOsProcs.addAll(procsDeCampos);
		todosOsProcs.addAll(procsDeCamposGrade);
		todosOsProcs.addAll(procsDeCamposOpo);
		todosOsProcs.add(procDeNome);
		todosOsProcs.add(procDeTexto);
		todosOsProcs.add(procDeURI);

		for (final ComponenteDix procurador : todosOsProcs)
			procurador.adicionarTratadorDeEventos(tratador);

	}

	public void mostrarCamposDeComponenteComOpcao() {
		for (final ComponenteDix campo : procsDeCamposOpo)
			campo.tornarVisivel();
	}

	public void mostrarCamposDeComponenteComTexto() {
		procDeTexto.tornarVisivel();
	}

	public void mostrarCamposDeComponenteComURI() {
		procDeURI.tornarVisivel();
	}

	public void mostrarCamposDeGrade() {
		for (final ComponenteDix campo : procsDeCamposGrade)
			campo.tornarVisivel();
	}

	public ComponenteDix obterCampoComNome(String nome) {

		return pgina.criarProcuradorDeComponente(nome);

	}

	public String obterNomeDoComponente() {
		return procDeNome.obterTexto();
	}

	public void ocultarCamposExtras() {
		for (final ComponenteDix campo : procsDeCamposGrade)
			campo.tornarInvisivel();
		for (final ComponenteDix campo : procsDeCamposOpo)
			campo.tornarInvisivel();

		procDeTexto.tornarInvisivel();
		procDeURI.tornarInvisivel();
	}

}
