package controle.mediador;

import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.PosicoesDeLegenda;
import infraestrutura.TiposDeComponentesGedix;
import infraestrutura.comandos.ComandoDeAtualizarAltura;
import infraestrutura.comandos.ComandoDeAtualizarColunas;
import infraestrutura.comandos.ComandoDeAtualizarEsquerda;
import infraestrutura.comandos.ComandoDeAtualizarHabilitacao;
import infraestrutura.comandos.ComandoDeAtualizarLargura;
import infraestrutura.comandos.ComandoDeAtualizarLegenda;
import infraestrutura.comandos.ComandoDeAtualizarLinhas;
import infraestrutura.comandos.ComandoDeAtualizarPosicaoDeLegenda;
import infraestrutura.comandos.ComandoDeAtualizarProfundidade;
import infraestrutura.comandos.ComandoDeAtualizarTexto;
import infraestrutura.comandos.ComandoDeAtualizarTopo;
import infraestrutura.comandos.ComandoDeAtualizarURI;
import infraestrutura.comandos.ComandoDeAtualizarVisibilidade;
import infraestrutura.comandos.ComandoNulo;
import infraestrutura.comandos.TipoComandoDeAtualizacao;
import infraestrutura.excecoes.ExcecaoDeArquivo;
import infraestrutura.excecoes.ExcecaoGedix;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import componentes.abstratos.TipoComponenteComURI;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.abstratos.opcao.TipoComponenteComOpcao;
import componentes.abstratos.texto.TipoComponenteComTexto;
import componentes.grade.TipoGrade;
import controle.mediador.tratadoresDeRede.TratadorDeAbrirArquivo;
import controle.mediador.tratadoresDeRede.TratadorDeCriarComponente;
import controle.mediador.tratadoresDeRede.TratadorDeCriarComponenteComDimensao;
import controle.mediador.tratadoresDeRede.TratadorDeEnviarComando;
import controle.mediador.tratadoresDeRede.TratadorDeGerarXML;
import controle.mediador.tratadoresDeRede.TratadorDeRemoverComponente;
import controle.mediador.tratadoresDeRede.TratadorDeSalvarArquivo;
import controle.mediador.tratadoresDeRede.TratadorDeTrocarComponenteEditado;
import controle.mediador.tratadoresDeRede.TratadorDeTrocarTipoDeCriacao;
import edugraf.jadix.ApliqueSemPichador;
import edugraf.jadix.fachada.GerenteDeRede;
import edugraf.jadix.primitivas.Primitiva;
import edugraf.jadix.tiposPrimitivos.Palavra;
import fachada.FachadaDoGedix;
import fachada.TipoFachadaDoGedix;


public class Mediador extends ApliqueSemPichador implements TipoMediador {

	private TipoFachadaDoGedix fachada;
	private GerenteDeRede gerente;
	private TipoComponenteGedix componenteSendoEditado;

	public void abrirArquivo(String endereo) {
		try {
			fachada.abrirPgina(endereo);
			colocarNoLog("Arquivo aberto com sucesso");
			atualizarAPartirDePginaAberta();
		} catch (final ExcecaoDeArquivo e) {
			colocarNoLog(e.getMessage());
		}

	}

	private void atualizarAPartirDePginaAberta() {
		gerente.dizer("\"limparPagina\"");
		Primitiva.descansar(0.5);
		gerente.dizer("\"limparListaDeComponentes\"");
		Primitiva.descansar(0.5);

		Primitiva.descansar(0.5);
		for (final TipoComponenteGedix componente : fachada.obterComponentes()) {
			pedirParaCriarComponenteDix(componente);
			registrarComponenteCriado(componente);
		}
	}

	private void colocarNoLog(String mensagem) {
		gerente.fixarParametro(mensagem).dizer("\"colocarNoLog\" texto");
	}

	public void criarComponente(int esquerda, int topo) {
		final TipoComponenteGedix componenteGedix = fachada.criarComponenteSelecionado(
				esquerda, topo);
		pedirParaCriarComponenteDix(componenteGedix);
		registrarComponenteCriado(componenteGedix);
	}

	public void criarComponente(int esquerda, int topo, int altura, int largura) {
		final TipoComponenteGedix componenteGedix = fachada.criarComponenteSelecionado(
				esquerda, topo, altura, largura);
		pedirParaCriarComponenteDix(componenteGedix);
		registrarComponenteCriado(componenteGedix);
	}

	public void enviarComandoDeAtualizao(String nomeDoAtributo, String valorDoAtributo) {
		try {
			fachada.atualizar(fabricarComandoDeAtualizacao(nomeDoAtributo,
					valorDoAtributo));

			gerente.fixarParametro(componenteSendoEditado.obterNome()).fixarParametro(
					nomeDoAtributo).fixarParametro(valorDoAtributo).dizer(
					"\"atualizarComponenteDix\" texto texto texto");
			colocarNoLog(componenteSendoEditado.obterNome() + " atualizado com sucesso");

		} catch (final ExcecaoGedix e) {
			colocarNoLog(e.getMessage());
		}

	}

	private TipoComandoDeAtualizacao fabricarComandoDeAtualizacao(String nomeDoAtributo,
			String valorDoAtributo) {

		final AtributosDosComponentesGedix atributo = AtributosDosComponentesGedix
				.valueOf(nomeDoAtributo);

		try {
			switch (atributo) {
			case altura:
				return new ComandoDeAtualizarAltura(componenteSendoEditado, Integer
						.parseInt(valorDoAtributo));
			case largura:
				return new ComandoDeAtualizarLargura(componenteSendoEditado, Integer
						.parseInt(valorDoAtributo));
			case topo:
				return new ComandoDeAtualizarTopo(componenteSendoEditado, Integer
						.parseInt(valorDoAtributo));
			case esquerda:
				return new ComandoDeAtualizarEsquerda(componenteSendoEditado, Integer
						.parseInt(valorDoAtributo));
			case profundidade:
				return new ComandoDeAtualizarProfundidade(componenteSendoEditado, Integer
						.parseInt(valorDoAtributo));
			case textoDaLegenda:
				return new ComandoDeAtualizarLegenda(componenteSendoEditado,
						valorDoAtributo);
			case colunas:
				return new ComandoDeAtualizarColunas((TipoGrade) componenteSendoEditado,
						Integer.parseInt(valorDoAtributo));
			case linhas:
				return new ComandoDeAtualizarLinhas((TipoGrade) componenteSendoEditado,
						Integer.parseInt(valorDoAtributo));
			case texto:
				return new ComandoDeAtualizarTexto(
						(TipoComponenteComTexto) componenteSendoEditado, valorDoAtributo);
			case uri:
				return new ComandoDeAtualizarURI(
						(TipoComponenteComURI) componenteSendoEditado, valorDoAtributo);
			case visvel:
				if (valorDoAtributo.equals("sim"))
					return new ComandoDeAtualizarVisibilidade(componenteSendoEditado,
							true);
				else
					return new ComandoDeAtualizarVisibilidade(componenteSendoEditado,
							false);

			case habilitado:
				if (valorDoAtributo.equals("esta"))
					return new ComandoDeAtualizarHabilitacao(componenteSendoEditado, true);
				else
					return new ComandoDeAtualizarHabilitacao(componenteSendoEditado,
							false);

			case posioDaLegenda:
				return new ComandoDeAtualizarPosicaoDeLegenda(componenteSendoEditado,
						PosicoesDeLegenda.valueOf(valorDoAtributo));
			}
		} catch (final NumberFormatException e) {
			gerente
					.dizer("\"colocarNoLog\" \"Insira somente nmeros em dimenses ou posio\"");
		}

		return new ComandoNulo();
	}

	public void gerarXML(String endereo) {
		try {
			fachada.gerarXML(endereo);
			colocarNoLog("XML criado em: " + endereo);
		} catch (final ExcecaoDeArquivo e) {
			colocarNoLog(e.getMessage());
		}
	}

	@Override
	public void iniciar() {
		fachada = FachadaDoGedix.instncia();
		gerente = obterGerenteDeRede();
		instalarTratadores();
	}

	private void instalarTratadores() {
		gerente.instalarTratador("\"trocarTipoDeCriacao\" texto",
				new TratadorDeTrocarTipoDeCriacao(this));

		gerente.instalarTratador("\"trocarComponenteDeEdicao\" texto",
				new TratadorDeTrocarComponenteEditado(this));

		gerente.instalarTratador("\"atualizarComponenteSelecionado\" texto texto",
				new TratadorDeEnviarComando(this));

		gerente.instalarTratador("\"criarComponente\" nmero nmero",
				new TratadorDeCriarComponente(this));

		gerente.instalarTratador("\"criarComponente\" nmero nmero nmero nmero",
				new TratadorDeCriarComponenteComDimensao(this));

		gerente.instalarTratador("\"removerComponenteSelecionado\"",
				new TratadorDeRemoverComponente(this));

		gerente.instalarTratador("\"abrirArquivo\" texto", new TratadorDeAbrirArquivo(
				this));
		gerente.instalarTratador("\"salvarArquivo\" texto", new TratadorDeSalvarArquivo(
				this));
		gerente.instalarTratador("\"gerarXML\" texto", new TratadorDeGerarXML(this));

	}

	private void pedirParaCriarComponenteDix(TipoComponenteGedix componenteGedix) {
		final Map<AtributosDosComponentesGedix, String> mapa = componenteGedix
				.obterMapeamentoAtributoValor();
		final List<Palavra> listaDePares = new ArrayList<Palavra>();

		for (final Entry<AtributosDosComponentesGedix, String> entrada : mapa.entrySet()) {
			final List<Palavra> par = new ArrayList<Palavra>();
			par.add(Palavra.criarTexto(entrada.getKey().toString()));
			par.add(Palavra.criarTexto(entrada.getValue()));
			listaDePares.add(Palavra.criarLista(par));
		}

		Primitiva.descansar(0.2);
		gerente.fixarParametro(listaDePares).dizer("\"criarComponenteDix\" lista");
	}

	private void preencherCamposDaJanelaDeEdio(TipoComponenteGedix componente) {
		final Map<AtributosDosComponentesGedix, String> mapa = componente
				.obterMapeamentoAtributoValor();

		for (final Entry<AtributosDosComponentesGedix, String> entrada : mapa.entrySet())
			gerente.fixarParametro(entrada.getKey().toString()).fixarParametro(
					entrada.getValue()).dizer("\"preencher\" texto texto");
	}

	private void registrarComponenteCriado(TipoComponenteGedix componente) {
		final String nome = componente.obterNome();

		gerente.fixarParametro(nome + " criado com sucesso!").dizer(
				"\"colocarNoLog\" texto");

		gerente.fixarParametro(nome).dizer("\"adicionarNaListaDeComponentes\" texto");
	}

	public void removerComponenteSelecionado() {
		if (componenteSendoEditado == null)
			gerente.dizer("\"colocarNoLog\" \"No h componente selecionado!\"");
		else {
			fachada.removerComponente(componenteSendoEditado);

			final String nome = componenteSendoEditado.obterNome();
			gerente.fixarParametro(nome).dizer("\"removerDaListaDeComponentes\" texto");
			gerente.fixarParametro(nome).dizer("\"removerComponenteDix\" texto");
			gerente.dizer("\"colocarNoLog\" \"Componente removido com sucesso\"");

			componenteSendoEditado = null;
		}
	}

	public void salvarArquivo(String endereo) {
		try {
			fachada.salvarPgina(endereo);
			colocarNoLog("Projeto Gedix salvo em: " + endereo);
		} catch (final ExcecaoDeArquivo e) {
			colocarNoLog(e.getMessage());
		}

	}

	public void trocarComponenteDeEdicao(String nomeDoComponente) {
		final TipoComponenteGedix componente = fachada.obterComponente(nomeDoComponente);
		componenteSendoEditado = componente;

		gerente.dizer("\"ocultarCamposExtras\"");

		if (componente instanceof TipoGrade)
			gerente.dizer("\"mostrarCamposDeGrade\"");
		if (componente instanceof TipoComponenteComURI)
			gerente.dizer("\"mostrarCamposDeURI\"");
		if (componente instanceof TipoComponenteComTexto)
			gerente.dizer("\"mostrarCamposDeTexto\"");
		if (componente instanceof TipoComponenteComOpcao)
			gerente.dizer("\"mostrarCamposDeOpcao\"");

		preencherCamposDaJanelaDeEdio(componente);
	}

	public void trocarTipoDeCriao(String nomeDoTipo) {
		final TiposDeComponentesGedix novoTipo = TiposDeComponentesGedix
				.valueOf(nomeDoTipo);
		fachada.trocarTipoDeCriao(novoTipo);
	}

}