package controle.novaPagina;

import static infraestrutura.AtributosDosComponentesGedix.altura;
import static infraestrutura.AtributosDosComponentesGedix.colunas;
import static infraestrutura.AtributosDosComponentesGedix.esquerda;
import static infraestrutura.AtributosDosComponentesGedix.habilitado;
import static infraestrutura.AtributosDosComponentesGedix.largura;
import static infraestrutura.AtributosDosComponentesGedix.linhas;
import static infraestrutura.AtributosDosComponentesGedix.nome;
import static infraestrutura.AtributosDosComponentesGedix.profundidade;
import static infraestrutura.AtributosDosComponentesGedix.texto;
import static infraestrutura.AtributosDosComponentesGedix.tipoDoComponente;
import static infraestrutura.AtributosDosComponentesGedix.topo;
import static infraestrutura.AtributosDosComponentesGedix.uri;
import static infraestrutura.AtributosDosComponentesGedix.visvel;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.TiposDeComponentesGedix;
import infraestrutura.geometria.Retangulo;
import infraestrutura.geometria.TipoPosicao;
import infraestrutura.geometria.TipoRetangulo;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import controle.novaPagina.tratadoresDeRede.TratadorDeAtualizarComponenteDix;
import controle.novaPagina.tratadoresDeRede.TratadorDeCriarComponenteDix;
import controle.novaPagina.tratadoresDeRede.TratadorDeLimparPagina;
import controle.novaPagina.tratadoresDeRede.TratadorDeRemoverComponenteDix;
import controle.novaPagina.tratadoresDix.TratadorDeDefinirAreaDeCriacao;
import edugraf.jadix.ApliqueSemPichador;
import edugraf.jadix.fachada.ComponenteDix;
import edugraf.jadix.fachada.GerenteDeRede;
import edugraf.jadix.fachada.PaginaDix;
import edugraf.jadix.fachada.TiposDeComponentesDix;
import edugraf.jadix.primitivas.Primitiva;


public class ControladorDaNovaPagina extends ApliqueSemPichador implements
		TipoControladorDaNovaPagina {

	PaginaDix pgina;
	private ComponenteDix reaSensvel;
	TipoPosicao posioInicial;
	TipoPosicao posioFinal;
	TipoRetangulo reaDeCriao;

	GerenteDeRede gerente;
	final List<ComponenteDix> listaDeComponentesDaPgina = new ArrayList<ComponenteDix>();

	public void atualizarComponenteDix(String nomeDoComponente, String nomeDoAtributo,
			String valor) {
		final ComponenteDix componenteAAtualizar = pgina
				.criarProcuradorDeComponente(nomeDoComponente);
		final AtributosDosComponentesGedix atributo = AtributosDosComponentesGedix
				.valueOf(nomeDoAtributo);

		try {
			switch (atributo) {
			case altura:
				componenteAAtualizar.fixarAltura(Integer.parseInt(valor));
				break;
			case largura:
				componenteAAtualizar.fixarLargura(Integer.parseInt(valor));
				break;
			case esquerda:
				componenteAAtualizar.fixarEsquerda(Integer.parseInt(valor));
				break;
			case topo:
				componenteAAtualizar.fixarTopo(Integer.parseInt(valor));
				break;
			// Aguardando entrega do Bug: 1967
			// case textoDaLegenda:
			// break;
			// Aguardando incluso de fixarPosioDeLegenda no ComponenteDix
			// case posioDaLegenda:
			// break;
			case profundidade:
				componenteAAtualizar.fixarProfundidade(Integer.parseInt(valor));
				break;
			case visvel:
				componenteAAtualizar.tornarVisivel();
				if (Boolean.parseBoolean(valor) == false)
					componenteAAtualizar.tornarInvisivel();
				break;
			case habilitado:
				componenteAAtualizar.habilitar();
				if (Boolean.parseBoolean(valor) == false)
					componenteAAtualizar.desabilitar();
				break;
			// Aguardando incluso de fixarLinhas e fixarColunas em
			// ComponenteDix
			// case linhas:
			// break;
			// case colunas:
			// break;
			case texto:
				componenteAAtualizar.fixarTexto(valor);
				break;
			case uri:
				componenteAAtualizar.fixarURI(valor);
				break;
			}
		} catch (final NumberFormatException e) {
		}

	}

	public void fixarPosioFinal(TipoPosicao posicao) {
		posioFinal = posicao;
		reaDeCriao = new Retangulo(posioInicial, posioFinal);
	}

	public void fixarPosioInicial(TipoPosicao posicao) {
		posioInicial = posicao;
	}

	void fixarValoresComuns(Map<AtributosDosComponentesGedix, String> valores,
			ComponenteDix componenteDix) {

		componenteDix.fixarAltura(Integer.parseInt(valores.get(altura)));
		componenteDix.fixarLargura(Integer.parseInt(valores.get(largura)));
		componenteDix.fixarTopo(Integer.parseInt(valores.get(topo)));
		componenteDix.fixarEsquerda(Integer.parseInt(valores.get(esquerda)));
		componenteDix.fixarProfundidade(Integer.parseInt(valores.get(profundidade)));

		if (Boolean.parseBoolean(valores.get(habilitado)) == false)
			componenteDix.desabilitar();

		if (Boolean.parseBoolean(valores.get(visvel)) == false)
			componenteDix.tornarInvisivel();
	}

	public void gerarComponenteDix(
			Map<AtributosDosComponentesGedix, String> valoresDoNovoComponente) {
		final TiposDeComponentesGedix tipo = TiposDeComponentesGedix
				.valueOf(valoresDoNovoComponente.get(tipoDoComponente));

		final String nomeDoComponente = valoresDoNovoComponente.get(nome);

		ComponenteDix componenteCriado = null;

		switch (tipo) {
		case boto:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.BOTO,
					nomeDoComponente);
			componenteCriado.fixarTexto(valoresDoNovoComponente.get(texto));
			break;

		case etiqueta:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.ETIQUETA,
					nomeDoComponente);
			componenteCriado.fixarTexto(valoresDoNovoComponente.get(texto));
			componenteCriado.fixarURI(valoresDoNovoComponente.get(uri));
			break;

		case caixaDeEscolha:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.CAIXA_DE_ESCOLHA, nomeDoComponente);
			break;

		case campoDeTexto:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.CAMPO_DE_TEXTO, nomeDoComponente);
			componenteCriado.fixarTexto(valoresDoNovoComponente.get(texto));
			break;

		case grade:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.GRADE,
					nomeDoComponente);
			final int numeroDeColunas = Integer.parseInt(valoresDoNovoComponente
					.get(colunas));
			final int numeroDeLinhas = Integer.parseInt(valoresDoNovoComponente
					.get(linhas));
			componenteCriado.fixarLinhasEColunas(numeroDeLinhas, numeroDeColunas);
			break;

		case reaDeTexto:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.REA_DE_TEXTO, nomeDoComponente);
			componenteCriado.fixarTexto(valoresDoNovoComponente.get(texto));
			break;

		case reaSensvel:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.REA_SENSVEL, nomeDoComponente);
			break;

		case imagem:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.IMAGEM,
					nomeDoComponente);
			componenteCriado.fixarURI(valoresDoNovoComponente.get(uri));
			break;

		case botesDeRdio:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.BOTES_DE_RDIO, nomeDoComponente);
			break;

		case listaDeEscolha:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.LISTA_DE_ESCOLHA, nomeDoComponente);
			break;

		}

		Primitiva.descansar(0.2);
		fixarValoresComuns(valoresDoNovoComponente, componenteCriado);
		listaDeComponentesDaPgina.add(componenteCriado);
	}

	public void iniciar() {
		pgina = obterPaginaDix();
		reaSensvel = pgina.criarProcuradorDeComponente("rea");
		gerente = obterGerenteDeRede();

		instalarTratadores();
	}

	private void instalarTratadores() {
		reaSensvel.adicionarTratadorDeEventos(new TratadorDeDefinirAreaDeCriacao(this));

		gerente.instalarTratador("\"criarComponenteDix\" lista",
				new TratadorDeCriarComponenteDix(this));
		gerente.instalarTratador("\"removerComponenteDix\" texto",
				new TratadorDeRemoverComponenteDix(this));
		gerente.instalarTratador("\"atualizarComponenteDix\" texto texto texto",
				new TratadorDeAtualizarComponenteDix(this));
		gerente.instalarTratador("\"limparPagina\"", new TratadorDeLimparPagina(this));
	}

	public void limparPgina() {
		for (final ComponenteDix componente : listaDeComponentesDaPgina)
			pgina.removerComponente(componente);
	}

	public void pedirCriaoDeComponente() {
		if (posioInicial.equals(posioFinal))
			gerente.fixarParametro(posioFinal.obterEsquerda()).fixarParametro(
					posioFinal.obterTopo()).dizer("\"criarComponente\" nmero nmero");
		else {
			final TipoPosicao posicao = reaDeCriao.obterPosio();

			gerente.fixarParametro(posicao.obterEsquerda()).fixarParametro(
					posicao.obterTopo()).fixarParametro(reaDeCriao.obterAltura())
					.fixarParametro(reaDeCriao.obterLargura()).dizer(
							"\"criarComponente\" nmero nmero nmero nmero");
		}

	}

	public void removerComponente(String nomeDoComponente) {
		final ComponenteDix componenteARemover = pgina
				.criarProcuradorDeComponente(nomeDoComponente);
		pgina.removerComponente(componenteARemover);
	}
}