package controle.novaPagina;

import static infraestrutura.AtributosDosComponentesGedix.altura;
import static infraestrutura.AtributosDosComponentesGedix.colunas;
import static infraestrutura.AtributosDosComponentesGedix.esquerda;
import static infraestrutura.AtributosDosComponentesGedix.habilitado;
import static infraestrutura.AtributosDosComponentesGedix.largura;
import static infraestrutura.AtributosDosComponentesGedix.linhas;
import static infraestrutura.AtributosDosComponentesGedix.nome;
import static infraestrutura.AtributosDosComponentesGedix.profundidade;
import static infraestrutura.AtributosDosComponentesGedix.texto;
import static infraestrutura.AtributosDosComponentesGedix.tipoDoComponente;
import static infraestrutura.AtributosDosComponentesGedix.topo;
import static infraestrutura.AtributosDosComponentesGedix.uri;
import static infraestrutura.AtributosDosComponentesGedix.visvel;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.TiposDeComponentesGedix;
import java.util.HashMap;
import java.util.Map;
import edugraf.jadix.fachada.ComponenteDix;
import edugraf.jadix.fachada.NotificavelPorDix;
import edugraf.jadix.fachada.PaginaDix;
import edugraf.jadix.fachada.TiposDeComponentesDix;


public class NovaPagina implements TipoNovaPagina {

	private final PaginaDix pgina;
	private final ComponenteDix reaSensvel;
	private final Map<String, ComponenteDix> tabelaDeComponentes = new HashMap<String, ComponenteDix>();

	public NovaPagina(PaginaDix novaPgina) {
		pgina = novaPgina;
		reaSensvel = pgina.criarProcuradorDeComponente("rea");
	}

	public void atualizarComponenteDix(String nome,
			AtributosDosComponentesGedix atributo, String valor) {

		final ComponenteDix componenteAAtualizar = tabelaDeComponentes.get(nome);

		try {
			switch (atributo) {
			case altura:
				componenteAAtualizar.fixarAltura(Integer.parseInt(valor));
				break;
			case largura:
				componenteAAtualizar.fixarLargura(Integer.parseInt(valor));
				break;
			case esquerda:
				componenteAAtualizar.fixarEsquerda(Integer.parseInt(valor));
				break;
			case topo:
				componenteAAtualizar.fixarTopo(Integer.parseInt(valor));
				break;
			// Aguardando entrega do Bug: 1967
			// case textoDaLegenda:
			// break;
			// Aguardando incluso de fixarPosioDeLegenda no ComponenteDix
			// case posioDaLegenda:
			// break;
			case profundidade:
				componenteAAtualizar.fixarProfundidade(Integer.parseInt(valor));
				break;
			case visvel:
				componenteAAtualizar.tornarVisivel();
				if (Boolean.parseBoolean(valor) == false)
					componenteAAtualizar.tornarInvisivel();
				break;
			case habilitado:
				componenteAAtualizar.habilitar();
				if (Boolean.parseBoolean(valor) == false)
					componenteAAtualizar.desabilitar();
				break;
			// Aguardando incluso de fixarLinhas e fixarColunas em
			// ComponenteDix
			// case linhas:
			// break;
			// case colunas:
			// break;
			case texto:
				componenteAAtualizar.fixarTexto(valor);
				break;
			case uri:
				componenteAAtualizar.fixarURI(valor);
				break;
			}
		} catch (final NumberFormatException e) {
		}

	}

	public void criarComponenteDix(Map<AtributosDosComponentesGedix, String> valores) {
		final TiposDeComponentesGedix tipo = TiposDeComponentesGedix.valueOf(valores
				.get(tipoDoComponente));

		final String nomeDoComponente = valores.get(nome);

		ComponenteDix componenteCriado = null;

		switch (tipo) {
		case boto:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.BOTO,
					nomeDoComponente);
			componenteCriado.fixarTexto(valores.get(texto));
			break;

		case etiqueta:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.ETIQUETA,
					nomeDoComponente);
			componenteCriado.fixarTexto(valores.get(texto));
			componenteCriado.fixarURI(valores.get(uri));
			break;

		case caixaDeEscolha:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.CAIXA_DE_ESCOLHA, nomeDoComponente);
			break;

		case campoDeTexto:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.CAMPO_DE_TEXTO, nomeDoComponente);
			componenteCriado.fixarTexto(valores.get(texto));
			break;

		case grade:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.GRADE,
					nomeDoComponente);
			final int numeroDeColunas = Integer.parseInt(valores.get(colunas));
			final int numeroDeLinhas = Integer.parseInt(valores.get(linhas));
			componenteCriado.fixarLinhasEColunas(numeroDeLinhas, numeroDeColunas);
			break;

		case reaDeTexto:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.REA_DE_TEXTO, nomeDoComponente);
			componenteCriado.fixarTexto(valores.get(texto));
			break;

		case reaSensvel:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.REA_SENSVEL, nomeDoComponente);
			break;

		case imagem:
			componenteCriado = pgina.criarComponente(TiposDeComponentesDix.IMAGEM,
					nomeDoComponente);
			componenteCriado.fixarURI(valores.get(uri));
			break;

		case botesDeRdio:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.BOTES_DE_RDIO, nomeDoComponente);
			break;

		case listaDeEscolha:
			componenteCriado = pgina.criarComponente(
					TiposDeComponentesDix.LISTA_DE_ESCOLHA, nomeDoComponente);
			break;

		}

		fixarValoresComuns(valores, componenteCriado);
	}

	private void fixarValoresComuns(Map<AtributosDosComponentesGedix, String> valores,
			ComponenteDix componenteDix) {

		componenteDix.fixarAltura(Integer.parseInt(valores.get(altura)));
		componenteDix.fixarLargura(Integer.parseInt(valores.get(largura)));
		componenteDix.fixarTopo(Integer.parseInt(valores.get(topo)));
		componenteDix.fixarEsquerda(Integer.parseInt(valores.get(esquerda)));
		componenteDix.fixarProfundidade(Integer.parseInt(valores.get(profundidade)));

		if (Boolean.parseBoolean(valores.get(habilitado)) == false)
			componenteDix.desabilitar();

		if (Boolean.parseBoolean(valores.get(visvel)) == false)
			componenteDix.tornarInvisivel();
	}

	public void instalarTratadorNaAreaSensivel(NotificavelPorDix tratador) {
		reaSensvel.adicionarTratadorDeEventos(tratador);
	}

	public void limpar() {
		for (final ComponenteDix c : tabelaDeComponentes.values())
			pgina.removerComponente(c);
	}

	public void removerComponenteDix(String nome) {
		pgina.removerComponente(tabelaDeComponentes.get(nome));
	}

}
