package fachada;

import infraestrutura.TiposDeComponentesGedix;
import infraestrutura.comandos.TipoComandoDeAtualizacao;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import infraestrutura.serializacao.SerializadorGedix;
import infraestrutura.serializacao.SerializadorXML;
import infraestrutura.serializacao.TipoSerializadorGedix;
import infraestrutura.serializacao.TipoSerializadorXML;
import java.util.Collection;
import pagina.PaginaGedix;
import pagina.TipoPaginaGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;


public class FachadaDoGedix implements TipoFachadaDoGedix {

	private static TipoFachadaDoGedix instncianica;

	public static TipoFachadaDoGedix instncia() {
		if (instncianica == null)
			instncianica = new FachadaDoGedix();

		return instncianica;
	}

	private final TipoFabrica fbricaDeComponentes;
	private TipoPaginaGedix pginaGedix;
	private final TipoSerializadorGedix serializadorGedix;
	private TiposDeComponentesGedix tipoDeCriao;
	private final TipoSerializadorXML serializadorXML;

	private FachadaDoGedix() {
		fbricaDeComponentes = FabricaDeComponentesGedix.instncia();
		pginaGedix = new PaginaGedix("pginaQualquer", "ttuloQualquer");
		tipoDeCriao = TiposDeComponentesGedix.boto;
		serializadorGedix = new SerializadorGedix();
		serializadorXML = new SerializadorXML();
	}

	public void abrirPgina(String endereo) {
		pginaGedix = serializadorGedix.recuperarPgina(endereo);
	}

	public void atualizar(TipoComandoDeAtualizacao comando) {
		comando.executar();
	}

	public TipoComponenteGedix criarComponenteSelecionado(int esquerda, int topo) {
		final TipoComponenteGedix c = fbricaDeComponentes.criarComponente(tipoDeCriao,
				esquerda, topo);

		pginaGedix.adicionarComponente(c);

		return c;
	}

	public TipoComponenteGedix criarComponenteSelecionado(int esquerda, int topo,
			int altura, int largura) {

		final TipoComponenteGedix c = fbricaDeComponentes.criarComponente(tipoDeCriao,
				esquerda, topo, largura, altura);

		pginaGedix.adicionarComponente(c);
		return c;
	}

	public void gerarXML(String endereo) {
		serializadorXML.salvarArquivoXML(pginaGedix, endereo);
	}

	public TipoComponenteGedix obterComponente(String nomeDoComponente) {
		return pginaGedix.obterComponente(nomeDoComponente);
	}

	public Collection<TipoComponenteGedix> obterComponentes() {
		return pginaGedix.obterComponentes();
	}

	public void removerComponente(TipoComponenteGedix componente) {
		pginaGedix.removerComponente(componente.obterNome());
	}

	public void salvarPgina(String endereo) {
		serializadorGedix.salvarPginaEmArquivo(pginaGedix, endereo);
	}

	public void trocarTipoDeCriao(TiposDeComponentesGedix novoTipo) {
		tipoDeCriao = novoTipo;
	}

}
