package infraestrutura.fabrica;

import infraestrutura.TiposDeComponentesGedix;
import proxies.ProxyDeAreaSensivel;
import proxies.ProxyDeGrade;
import proxies.ProxyDeImagem;
import proxies.comopcao.ProxyDeBotoesDeRadio;
import proxies.comopcao.ProxyDeCaixaDeEscolha;
import proxies.comopcao.ProxyDeListaDeEscolha;
import proxies.comtexto.ProxyDeAreaDeTexto;
import proxies.comtexto.ProxyDeBotao;
import proxies.comtexto.ProxyDeCampoDeTexto;
import proxies.comtexto.ProxyDeEtiqueta;
import componentes.AreaSensivel;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.comopcao.BotoesDeRadio;
import componentes.comopcao.CaixaDeEscolha;
import componentes.comopcao.ListaDeEscolha;
import componentes.comtexto.AreaDeTexto;
import componentes.comtexto.Botao;
import componentes.comtexto.CampoDeTexto;
import componentes.comtexto.etiqueta.Etiqueta;
import componentes.grade.Grade;
import componentes.imagem.Imagem;


public class FabricaDeComponentesGedix implements TipoFabrica {

	private static TipoFabrica instncianica;

	public static TipoFabrica instncia() {
		if (instncianica == null)
			instncianica = new FabricaDeComponentesGedix();

		return instncianica;
	}

	private FabricaDeComponentesGedix() {
	}

	public TipoComponenteGedix criarComponente(TiposDeComponentesGedix tipo,
			int esquerda, int topo) {
		final TipoComponenteGedix c = retornarProxyComNome(tipo);
		c.fixarEsquerda(esquerda);
		c.fixarTopo(topo);

		return c;
	}

	public TipoComponenteGedix criarComponente(TiposDeComponentesGedix tipo,
			int esquerda, int topo, int largura, int altura) {
		final TipoComponenteGedix c = criarComponente(tipo, esquerda, topo);
		c.fixarLargura(largura);
		c.fixarAltura(altura);

		return c;
	}

	protected TipoComponenteGedix retornarProxyComNome(TiposDeComponentesGedix tipo) {
		switch (tipo) {
		case etiqueta:
			return new ProxyDeEtiqueta(new Etiqueta());
		case campoDeTexto:
			return new ProxyDeCampoDeTexto(new CampoDeTexto());
		case reaDeTexto:
			return new ProxyDeAreaDeTexto(new AreaDeTexto());
		case reaSensvel:
			return new ProxyDeAreaSensivel(new AreaSensivel());
		case grade:
			return new ProxyDeGrade(new Grade());
		case imagem:
			return new ProxyDeImagem(new Imagem());
		case caixaDeEscolha:
			return new ProxyDeCaixaDeEscolha(new CaixaDeEscolha());
		case listaDeEscolha:
			return new ProxyDeListaDeEscolha(new ListaDeEscolha());
		case botesDeRdio:
			return new ProxyDeBotoesDeRadio(new BotoesDeRadio());
		default:
			return new ProxyDeBotao(new Botao());
		}
	}

}
