package pagina;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import componentes.abstratos.gedix.TipoComponenteGedix;


public class PaginaGedix implements TipoPaginaGedix {

	private static final long serialVersionUID = 1L;
	private final Map<String, TipoComponenteGedix> tabelaDeComponentes = new HashMap<String, TipoComponenteGedix>();
	private final String nome;
	private final String ttulo;

	public PaginaGedix(String nomeDaPagina, String ttuloDaPagina) {
		nome = nomeDaPagina;
		ttulo = ttuloDaPagina;
	}

	public void adicionarComponente(TipoComponenteGedix componente) {
		tabelaDeComponentes.put(componente.obterNome(), componente);
	}

	public boolean equals(Object outra) {
		if (outra instanceof PaginaGedix) {
			final PaginaGedix outraPgina = (PaginaGedix) outra;
			return obterComponentes().equals(outraPgina.obterComponentes());
		}
		return false;
	}

	public String gerarXML() {

		return "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
				+ "\n"
				+ "<?xml-stylesheet type=\"text/xsl\" href=\"/dix/dix_2007-10-23/transformacao/dix.xml\"?>"
				+ "\n\n" + "<pgina nome=\"" + nome + "\" ttulo=\"" + ttulo
				+ "\" tema=\"campeche\">" + "\n\n" + obterXMLDosComponentes()
				+ "</pgina>";

	}

	public int hashCode() {
		return tabelaDeComponentes.hashCode();
	}

	public TipoComponenteGedix obterComponente(String nomeDoComponente) {
		return tabelaDeComponentes.get(nomeDoComponente);

	}

	public Set<TipoComponenteGedix> obterComponentes() {
		return new HashSet<TipoComponenteGedix>(tabelaDeComponentes.values());
	}

	public String obterNome() {
		return nome;
	}

	private String obterXMLDosComponentes() {
		String xmlDosComponentes = "";

		for (final TipoComponenteGedix componente : tabelaDeComponentes.values())
			xmlDosComponentes = xmlDosComponentes + componente.gerarXML() + "\n\n";
		return xmlDosComponentes;
	}

	public void removerComponente(String nomeDoComponente) {
		tabelaDeComponentes.remove(nomeDoComponente);

	}

	public String toString() {
		String repr = "";
		for (final TipoComponenteGedix c : obterComponentes())
			repr = repr + c.toString() + "\n";

		return repr;
	}

}
