package proxies.abstratos;

import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.PosicoesDeLegenda;
import infraestrutura.TiposDeComponentesGedix;
import infraestrutura.excecoes.DimensaoInvalida;
import infraestrutura.excecoes.PosicaoInvalida;
import java.util.Map;
import componentes.abstratos.gedix.TipoComponenteGedix;


public abstract class ProxyDeComponenteGedix implements TipoComponenteGedix {

	protected TipoComponenteGedix componente;

	public ProxyDeComponenteGedix(TipoComponenteGedix c) {
		componente = c;
	}

	public void desabilitar() {
		componente.desabilitar();
	}

	public boolean equals(Object outro) {
		return componente.equals(outro);
	}

	public boolean estHabilitado() {
		return componente.estHabilitado();
	}

	public boolean estVisvel() {
		return componente.estVisvel();
	}

	public void fixarAltura(int novaAltura) {
		if (novaAltura <= 0)
			throw new DimensaoInvalida();
		componente.fixarAltura(novaAltura);
	}

	public void fixarEsquerda(int novaEsquerda) {
		if (novaEsquerda < 0)
			throw new PosicaoInvalida();
		componente.fixarEsquerda(novaEsquerda);
	}

	public void fixarLargura(int novaLargura) {
		if (novaLargura <= 0)
			throw new DimensaoInvalida();
		componente.fixarLargura(novaLargura);
	}

	public void fixarLegenda(String novaLegenda) {
		componente.fixarLegenda(novaLegenda);
	}

	public void fixarPosioDeLegenda(PosicoesDeLegenda novaPosio) {
		componente.fixarPosioDeLegenda(novaPosio);
	}

	public void fixarProfundidade(int novaProfundidade) {
		componente.fixarProfundidade(novaProfundidade);
	}

	public void fixarTopo(int novoTopo) {
		if (novoTopo < 0)
			throw new PosicaoInvalida();
		componente.fixarTopo(novoTopo);
	}

	public String gerarXML() {
		return componente.gerarXML();
	}

	public void habilitar() {
		componente.habilitar();
	}

	public int hashCode() {
		return componente.hashCode();
	}

	public int obterAltura() {
		return componente.obterAltura();
	}

	public int obterEsquerda() {
		return componente.obterEsquerda();
	}

	public int obterLargura() {
		return componente.obterLargura();
	}

	public String obterLegenda() {
		return componente.obterLegenda();
	}

	public Map<AtributosDosComponentesGedix, String> obterMapeamentoAtributoValor() {
		return componente.obterMapeamentoAtributoValor();
	}

	public String obterNome() {
		return componente.obterNome();
	}

	public PosicoesDeLegenda obterPosioDeLegenda() {
		return componente.obterPosioDeLegenda();
	}

	public int obterProfundidade() {
		return componente.obterProfundidade();
	}

	public TiposDeComponentesGedix obterTipo() {
		return componente.obterTipo();
	}

	public int obterTopo() {
		return componente.obterTopo();
	}

	public void tornarInvisvel() {
		componente.tornarInvisvel();
	}

	public void tornarVisvel() {
		componente.tornarVisvel();
	}

	public String toString() {
		return componente.toString();
	}
}
