package testes.componentes;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.TiposDeComponentesGedix;
import java.util.Map;
import org.junit.Before;
import org.junit.Test;
import testes.componentes.abstratos.TestesDeComponentesGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.grade.Grade;
import componentes.grade.TipoGrade;


public class TestesDeGrade extends TestesDeComponentesGedix {

	private TipoGrade grade;

	protected TipoComponenteGedix componenteGedix() {
		return grade;
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarONmeroDeColunasComValorNegativo() {
		grade.fixarColunas(-1);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarONmeroDeColunasComValorZero() {
		grade.fixarColunas(0);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarONmeroDeLinhasComValorNegativo() {
		grade.fixarLinhas(-3);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarONmeroDeLinhasComValorZero() {
		grade.fixarLinhas(0);
	}

	@Test
	public void oMapeamentoAtributoValorDaGradeContmLinhasEColunas() {
		final String linhas = String.valueOf(grade.obterLinhas());
		final String colunas = String.valueOf(grade.obterColunas());

		final Map<AtributosDosComponentesGedix, String> mapeamento = grade
				.obterMapeamentoAtributoValor();

		assertEquals(linhas, mapeamento.get(AtributosDosComponentesGedix.linhas));
		assertEquals(colunas, mapeamento.get(AtributosDosComponentesGedix.colunas));
	}

	@Test
	public void oNmeroDeColunasInicialTrs() {
		assertEquals(3, grade.obterColunas());
	}

	@Test
	public void oNmeroDeLinhasInicialTrs() {
		assertEquals(3, grade.obterLinhas());
	}

	@Test
	public void oTipoDeUmaGradeOElementoEnumGrade() {
		assertEquals(TiposDeComponentesGedix.grade, grade.obterTipo());
	}

	@Test
	public void oXMLDaGradeContmSuasLinhasEColunas() {

		grade.fixarColunas(7);
		grade.fixarLinhas(9);

		final String textoXML = grade.gerarXML();

		assertTrue(textoXML.contains("linhas=\"9\""));
		assertTrue(textoXML.contains("colunas=\"7\""));
	}

	@Before
	public void prepararTestes() {
		grade = new Grade();
	}

	@Test
	public void seONmeroDeColunasForFixadoEleSeAltera() {
		grade.fixarColunas(5);
		assertEquals(5, grade.obterColunas());
	}

	@Test
	public void seONmeroDeLinhasForFixadoEleSeAltera() {
		grade.fixarLinhas(5);
		assertEquals(5, grade.obterLinhas());
	}

	@Test
	public void umaGradeContmSuasLinhasEColunasNoToString() {
		final String repr = grade.toString();

		assertTrue(repr.contains("linhas=" + grade.obterLinhas()));
		assertTrue(repr.contains("colunas=" + grade.obterColunas()));
	}

}
