package testes.componentes;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.TiposDeComponentesGedix;
import java.util.Map;
import org.junit.Before;
import org.junit.Test;
import testes.componentes.abstratos.TestesDeComponentesGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.imagem.Imagem;
import componentes.imagem.TipoImagem;


public class TestesDeImagem extends TestesDeComponentesGedix {

	private TipoImagem imagem;

	@Test
	public void aoFixarSeAURIdeUmaimagemElaMuda() {
		imagem.fixarURI("imagens/nada.jpg");
		assertEquals("imagens/nada.jpg", imagem.obterURI());
	}

	protected TipoComponenteGedix componenteGedix() {
		return imagem;
	}

	@Test
	public void oMapeamentoAtributoValorDeUmaImagemContmSuaURI() {
		final Map<AtributosDosComponentesGedix, String> mapeamento = imagem
				.obterMapeamentoAtributoValor();

		assertEquals(imagem.obterURI(), mapeamento.get(AtributosDosComponentesGedix.uri));
	}

	@Test
	public void oTipoDeUmaImagemOElementoEnumImagem() {
		assertEquals(TiposDeComponentesGedix.imagem, imagem.obterTipo());
	}

	@Test
	public void oXMLDaImagemSempreContmURI() {
		imagem.fixarURI("");

		final String textoXMLSemURI = imagem.gerarXML();
		assertTrue(textoXMLSemURI.contains("uri=\"\""));
	}

	@Before
	public void prepararTestes() {
		imagem = new Imagem();
	}

	@Test
	public void umaImagemContmSuaURINoToString() {
		final String repr = imagem.toString();

		assertTrue(repr.contains("uri=" + imagem.obterURI()));
	}

	@Test
	public void umaImagemTemUmaURIInicialNula() {
		assertEquals(null, imagem.obterURI());
	}

}
