package testes.componentes.abstratos;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import java.util.Collection;
import org.junit.Test;
import componentes.Opcao;
import componentes.TipoOpcao;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.abstratos.opcao.TipoComponenteComOpcao;


public abstract class TestesDeComponentesComOpcao extends TestesDeComponentesGedix {

	@Test
	public void aoAdicionarUmaOpoUmComponenteComOpoEleAContm() {
		final TipoOpcao opo = new Opcao("novaOpo");
		componenteComOpo().adicionarOpo(opo);

		assertTrue(componenteComOpo().obterOpes().contains(opo));
	}

	protected abstract TipoComponenteComOpcao componenteComOpo();

	protected TipoComponenteGedix componenteGedix() {
		return componenteComOpo();
	}

	@Test
	public void podeSeRemoverUmaOpoDoComponenteComOpoUsandoSeuNome() {
		final TipoOpcao opao1 = new Opcao("Opo1");
		final TipoOpcao opao2 = new Opcao("Opo2");

		componenteComOpo().adicionarOpo(opao1);
		componenteComOpo().adicionarOpo(opao2);

		componenteComOpo().removerOpo(opao1.obterNome());

		final Collection<TipoOpcao> opesDoComponente = componenteComOpo()
				.obterOpes();

		assertFalse(opesDoComponente.contains(opao1));
		assertTrue(opesDoComponente.contains(opao2));
	}

	@Test
	public void seAdicionarUmaOpoAUmComponenteOXMLContmOTextoDaOpo() {
		componenteComOpo().adicionarOpo(new Opcao("textoDaOpo"));

		final String textoXML = componenteComOpo().gerarXML();

		assertTrue(textoXML.contains(">textoDaOpo<"));
	}

	@Test
	public void umComponenteComOpoInicialmenteEstVazio() {
		assertTrue(componenteComOpo().obterOpes().isEmpty());
	}

	@Test
	public void umComponenteQueTemUmaOpcaoTemOpesNoSeuXML() {
		componenteComOpo().adicionarOpo(new Opcao("textoDaOpo"));

		final String textoXML = componenteComOpo().gerarXML();

		assertTrue(textoXML.contains("<opo"));

	}
}
