package testes.componentes.abstratos;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import infraestrutura.AtributosDosComponentesGedix;
import infraestrutura.PosicoesDeLegenda;
import infraestrutura.TiposDeComponentesGedix;
import java.util.Map;
import org.junit.Test;
import componentes.abstratos.gedix.TipoComponenteGedix;


public abstract class TestesDeComponentesGedix {

	@Test
	public void aAlturaPadroDeUmComponenteCinco() {
		assertEquals(5, componenteGedix().obterAltura());
	}

	@Test
	public void aLarguraPadroDeUmComponenteCinco() {
		assertEquals(5, componenteGedix().obterLargura());
	}

	@Test
	public void aLegendaInicialDeUmComponenteVazia() {
		assertEquals("", componenteGedix().obterLegenda());
	}

	@Test
	public void aoDesabilitarUmComponenteEleSeTornaDesabilitado() {
		componenteGedix().desabilitar();
		assertFalse(componenteGedix().estHabilitado());
	}

	@Test
	public void aoFixarNovaProfundidadeDeUmComponenteElaMuda() {
		componenteGedix().fixarProfundidade(5);
		assertEquals(5, componenteGedix().obterProfundidade());
	}

	@Test
	public void aoHabilitarUmComponenteDesabilitadoEleSeTornaHabilitado() {
		componenteGedix().desabilitar();
		componenteGedix().habilitar();
		assertTrue(componenteGedix().estHabilitado());
	}

	@Test
	public void aPosioDeLegendaPadroDeUmComponenteemCima() {
		assertEquals(PosicoesDeLegenda.emCima, componenteGedix().obterPosioDeLegenda());
	}

	protected abstract TipoComponenteGedix componenteGedix();

	@Test(expected = AssertionError.class)
	public void noPossvelFixarAAlturaComValorNegativo() {
		componenteGedix().fixarAltura(-1);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarAAlturaComValorZero() {
		componenteGedix().fixarAltura(0);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarAEsquerdaComValorNegativo() {
		componenteGedix().fixarEsquerda(-40);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarALarguraComValorNegativo() {
		componenteGedix().fixarLargura(-1);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarALarguraComValorZero() {
		componenteGedix().fixarLargura(0);
	}

	@Test(expected = AssertionError.class)
	public void noPossvelFixarOTopoComValorNegativo() {
		componenteGedix().fixarTopo(-1);
	}

	@Test
	public void oValorHabilitadoPadroVerdadeiro() {
		assertTrue(componenteGedix().estHabilitado());
	}

	@Test
	public void podeSeMudarAPosioDeLegendaDeUmComponente() {
		componenteGedix().fixarPosioDeLegenda(PosicoesDeLegenda.Esquerda);
		assertEquals(PosicoesDeLegenda.Esquerda, componenteGedix()
				.obterPosioDeLegenda());
	}

	@Test
	public void seFixarAAlturaDeUmComponenteElaSeTornaOValorFixado() {
		componenteGedix().fixarAltura(16);
		assertEquals(16, componenteGedix().obterAltura());
	}

	@Test
	public void seFixarAEsquerdaDeUmComponenteElaSeTornaOValorFixado() {
		componenteGedix().fixarEsquerda(100);
		assertEquals(100, componenteGedix().obterEsquerda());
	}

	@Test
	public void seFixarALarguraDeUmComponenteElaSeTornaOValorFixado() {
		componenteGedix().fixarLargura(27);
		assertEquals(27, componenteGedix().obterLargura());
	}

	@Test
	public void seFixarALegendaDeUmComponenteElaSeALtera() {
		componenteGedix().fixarLegenda("legendaNova");
		assertEquals("legendaNova", componenteGedix().obterLegenda());
	}

	@Test
	public void seFixarOTopoDeUmComponenteEleSeTornaOValorFixado() {
		componenteGedix().fixarTopo(50);
		assertEquals(50, componenteGedix().obterTopo());
	}

	@Test
	public void seTornarUmComponenteInvisvelEleDeixaDeEstarVisvel() {
		componenteGedix().tornarInvisvel();
		assertFalse(componenteGedix().estVisvel());
	}

	@Test
	public void umComponentePodeCriarUmXMLComCamposEspecficos() {
		final String textoXML = componenteGedix().gerarXML();

		assertTrue(textoXML.contains("nome="));
		assertTrue(textoXML.contains("topo="));
		assertTrue(textoXML.contains("esquerda="));
		assertTrue(textoXML.contains("altura="));
		assertTrue(textoXML.contains("largura="));
		assertTrue(textoXML.contains("legenda="));
		assertTrue(textoXML.contains("posioDaLegenda"));
		assertTrue(textoXML.contains("visvel="));
		assertTrue(textoXML.contains("profundidade="));
		assertTrue(textoXML.contains("habilitado"));
	}

	@Test
	public void umComponenteRecmCriadoVisvel() {
		assertTrue(componenteGedix().estVisvel());
	}

	@Test
	public void umComponenteTemSeuMapeamentoAtributoValorCoerenteComSeuEstado() {
		final String nome = componenteGedix().obterNome();
		final String tipo = componenteGedix().obterTipo().toString();
		final String esquerda = String.valueOf(componenteGedix().obterEsquerda());
		final String topo = String.valueOf(componenteGedix().obterTopo());
		final String largura = String.valueOf(componenteGedix().obterLargura());
		final String altura = String.valueOf(componenteGedix().obterAltura());
		final String profundidade = String.valueOf(componenteGedix().obterProfundidade());
		final String visvel = String.valueOf(componenteGedix().estVisvel());
		final String habilitado = String.valueOf(componenteGedix().estHabilitado());
		final String legenda = componenteGedix().obterLegenda();
		final String posioDaLegenda = componenteGedix().obterPosioDeLegenda()
				.toString();

		final Map<AtributosDosComponentesGedix, String> mapaValorAtributo = componenteGedix()
				.obterMapeamentoAtributoValor();

		assertEquals(nome, mapaValorAtributo.get(AtributosDosComponentesGedix.nome));
		assertEquals(tipo, mapaValorAtributo
				.get(AtributosDosComponentesGedix.tipoDoComponente));
		assertEquals(esquerda, mapaValorAtributo
				.get(AtributosDosComponentesGedix.esquerda));
		assertEquals(topo, mapaValorAtributo.get(AtributosDosComponentesGedix.topo));
		assertEquals(altura, mapaValorAtributo.get(AtributosDosComponentesGedix.largura));
		assertEquals(largura, mapaValorAtributo.get(AtributosDosComponentesGedix.altura));
		assertEquals(profundidade, mapaValorAtributo
				.get(AtributosDosComponentesGedix.profundidade));
		assertEquals(visvel, mapaValorAtributo.get(AtributosDosComponentesGedix.visvel));
		assertEquals(habilitado, mapaValorAtributo
				.get(AtributosDosComponentesGedix.habilitado));
		assertEquals(legenda, mapaValorAtributo
				.get(AtributosDosComponentesGedix.textoDaLegenda));
		assertEquals(posioDaLegenda, mapaValorAtributo
				.get(AtributosDosComponentesGedix.posioDaLegenda));

	}

	@Test
	public void umComponenteTemSeusValoresBsicosNoToString() {
		final String repr = componenteGedix().toString();

		assertTrue(repr.contains("esquerda=" + componenteGedix().obterEsquerda()));
		assertTrue(repr.contains("topo=" + componenteGedix().obterTopo()));
		assertTrue(repr.contains("largura=" + componenteGedix().obterLargura()));
		assertTrue(repr.contains("altura=" + componenteGedix().obterAltura()));
		assertTrue(repr.contains("nome=" + componenteGedix().obterNome()));
		assertTrue(repr.contains("tipoDoComponente=" + componenteGedix().obterTipo()));
		assertTrue(repr.contains("textoDaLegenda=" + componenteGedix().obterLegenda()));
		assertTrue(repr.contains("posioDaLegenda="
				+ componenteGedix().obterPosioDeLegenda()));
		assertTrue(repr.contains("profundidade=" + componenteGedix().obterProfundidade()));
		assertTrue(repr.contains("visvel=" + componenteGedix().estVisvel()));
		assertTrue(repr.contains("habilitado=" + componenteGedix().estHabilitado()));
	}

	@Test
	public void umComponenteTemSeuXMLIniciadoPeloSeuTipo() {
		final String XMLDoComponente = componenteGedix().gerarXML();
		final TiposDeComponentesGedix seuTipo = componenteGedix().obterTipo();

		assertTrue(XMLDoComponente.startsWith("<" + seuTipo));
	}

}