package testes.fachada;

import static infraestrutura.TiposDeComponentesGedix.grade;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;
import infraestrutura.comandos.ComandoDeAtualizarTopo;
import infraestrutura.comandos.TipoComandoDeAtualizacao;
import infraestrutura.excecoes.ExcecaoDeArquivo;
import infraestrutura.excecoes.PosicaoInvalida;
import java.util.HashSet;
import java.util.Set;
import org.junit.Before;
import org.junit.Test;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.grade.TipoGrade;
import fachada.FachadaDoGedix;
import fachada.TipoFachadaDoGedix;


public class TestesDaFachadaDoGedix {

	private TipoFachadaDoGedix fachada;

	@Test
	public void aFachadaCapazDeSerializarUmaPginaGedix() {
		final TipoComponenteGedix componenteQualquer = fachada
				.criarComponenteSelecionado(23, 45);
		final TipoComponenteGedix outroComponenteQualquer = fachada
				.criarComponenteSelecionado(34, 12, 76, 32);

		final Set<TipoComponenteGedix> componentes = new HashSet<TipoComponenteGedix>();
		componentes.addAll(fachada.obterComponentes());
		fachada.salvarPgina(System.getProperty("user.home")
				+ System.getProperty("file.separator") + "pagina.ged");

		fachada.removerComponente(componenteQualquer);
		fachada.removerComponente(outroComponenteQualquer);

		fachada.abrirPgina(System.getProperty("user.home")
				+ System.getProperty("file.separator") + "pagina.ged");
		final Set<TipoComponenteGedix> componentesDaPginaAberta = new HashSet<TipoComponenteGedix>();
		componentesDaPginaAberta.addAll(fachada.obterComponentes());

		assertEquals(componentes, componentesDaPginaAberta);
	}

	@Test
	public void aFachadaPodeCriarUmComponenteAPartirDeUmaPosicao() {
		final int esquerda = 50;
		final int topo = 150;

		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(
				esquerda, topo);
		assertEquals(esquerda, componente.obterEsquerda());
		assertEquals(topo, componente.obterTopo());
	}

	@Test
	public void aFachadaPodeCriarUmComponenteAPartirDeUmaPosicaoEDimenso() {
		final int esquerda = 50;
		final int topo = 150;
		final int altura = 20;
		final int largura = 30;

		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(
				esquerda, topo, altura, largura);

		assertEquals(esquerda, componente.obterEsquerda());
		assertEquals(topo, componente.obterTopo());
		assertEquals(largura, componente.obterLargura());
		assertEquals(altura, componente.obterAltura());
	}

	@Test
	public void aFachadaPodeTrocarDeTipoDeCriao() {
		fachada.trocarTipoDeCriao(grade);
		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(10, 10);

		assertTrue(componente instanceof TipoGrade);
	}

	@Test(expected = PosicaoInvalida.class)
	public void casoAFachadaRecebaUmComandoInvlidoElaRetornaUmaExceo() {
		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(20, 20);
		final TipoComandoDeAtualizacao comando = new ComandoDeAtualizarTopo(componente,
				-2);

		fachada.atualizar(comando);

		assertEquals(20, componente.obterTopo());
	}

	@Before
	public void prepararTestes() {
		fachada = FachadaDoGedix.instncia();
	}

	@Test
	public void quandoAFachadaCriaUmComponenteEleFicaNaColeoDosComponentes() {
		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(10, 15);

		assertTrue(fachada.obterComponentes().contains(componente));
	}

	@Test
	public void quandoAFachadaRecebeUmComandoDeAtualizaoElaOAplica() {
		final TipoComponenteGedix componente = fachada.criarComponenteSelecionado(20, 20);
		final TipoComandoDeAtualizacao comando = new ComandoDeAtualizarTopo(componente,
				25);

		fachada.atualizar(comando);

		assertEquals(25, componente.obterTopo());
	}

	@Test(expected = ExcecaoDeArquivo.class)
	public void seAbrirUmaPginaQueNoExisteAFachadaRetornaExceoEMantmAPginaAnterior() {
		final Set<TipoComponenteGedix> componentesAntesDeAbrir = new HashSet<TipoComponenteGedix>(
				fachada.obterComponentes());

		fachada.abrirPgina("/endereco/Que/Non/Ecziste");

		final Set<TipoComponenteGedix> componentesApsAbrir = new HashSet<TipoComponenteGedix>(
				fachada.obterComponentes());

		assertEquals(componentesAntesDeAbrir, componentesApsAbrir);
	}

	@Test
	public void sExisteUmanicaFachadaDoGedix() {
		final TipoFachadaDoGedix fachada2 = FachadaDoGedix.instncia();

		assertSame(fachada, fachada2);
	}

}
