package testes.infraestrutura;

import static infraestrutura.TiposDeComponentesGedix.campoDeTexto;
import static infraestrutura.TiposDeComponentesGedix.etiqueta;
import static infraestrutura.TiposDeComponentesGedix.grade;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;
import infraestrutura.excecoes.PosicaoInvalida;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import org.junit.Before;
import org.junit.Test;
import proxies.ProxyDeGrade;
import componentes.abstratos.gedix.TipoComponenteGedix;


public class TestesDeFabrica {

	private TipoFabrica fabrica;

	@Test(expected = PosicaoInvalida.class)
	public void casoAFbricaRecebaUmaPosioInvlidaElaLanaUmaExceo() {
		fabrica.criarComponente(campoDeTexto, -1, -1);
	}

	@Test
	public void deveSeDizerFbricaNoPedidoQualTipoDeComponenteElaDeveCriar() {
		final TipoComponenteGedix componente = fabrica.criarComponente(grade, 20, 15);

		assertNotNull(componente);
		assertTrue(componente instanceof ProxyDeGrade);
	}

	@Test
	public void existeUmanicaFbricaDeComponentesGedix() {
		final TipoFabrica fabrica2 = FabricaDeComponentesGedix.instncia();

		assertSame(fabrica, fabrica2);
	}

	@Before
	public void prepararTestes() {
		fabrica = FabricaDeComponentesGedix.instncia();
	}

	@Test
	public void seAFabricaReceberDimensaoElaCriaOComponenteComEstaDimenso() {
		final TipoComponenteGedix componente = fabrica.criarComponente(campoDeTexto, 0,
				0, 15, 10);

		assertEquals(15, componente.obterLargura());
		assertEquals(10, componente.obterAltura());
	}

	@Test
	public void umaFabricaCriaUmComponenteComTopoEEsquerdaDados() {
		final TipoComponenteGedix componente = fabrica.criarComponente(etiqueta, 20, 15);

		assertEquals(20, componente.obterEsquerda());
		assertEquals(15, componente.obterTopo());
	}

}
