package testes.infraestrutura;

import static infraestrutura.TiposDeComponentesGedix.boto;
import static infraestrutura.TiposDeComponentesGedix.etiqueta;
import static org.junit.Assert.assertEquals;
import infraestrutura.excecoes.ExcecaoDeArquivo;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import infraestrutura.serializacao.SerializadorGedix;
import infraestrutura.serializacao.TipoSerializadorGedix;
import org.junit.Before;
import org.junit.Test;
import pagina.PaginaGedix;
import pagina.TipoPaginaGedix;


public class TestesDeSerializadorGedix {

	private TipoSerializadorGedix serializador;
	private TipoFabrica fabrica;
	private TipoPaginaGedix pgina;

	@Test(expected = ExcecaoDeArquivo.class)
	public void aoSeTentarRecuperarUmaPginaDeUmCaminhoInexistenteLanadaUmaExceoDeArquivo() {
		serializador.recuperarPgina("/wwwyx");
	}

	@Test(expected = ExcecaoDeArquivo.class)
	public void aoSeTentarSalvarUmaPginaNumCaminhoNoPermitidoLanadaUmaExceoDeArquivo() {
		serializador.salvarPginaEmArquivo(pgina, "/kdjfu");
	}

	@Test(expected = ExcecaoDeArquivo.class)
	public void aoSeTentarSalvarUmaPginaSemAPermissoAdequadaUmaExceoDeArquivoLanada() {
		if (System.getProperty("os.name").toLowerCase().contains("linux"))
			if (!System.getProperty("user.name").equals("root"))
				serializador.salvarPginaEmArquivo(pgina, "/usr/bin/pagina1.ged");
	}

	@Before
	public void prepararTestes() {
		serializador = new SerializadorGedix();
		fabrica = FabricaDeComponentesGedix.instncia();
		pgina = new PaginaGedix("paginaBoa", "uma pgina boa");
	}

	@Test
	public void umaPginaRecuperadaPeloSerializadorIgualOriginal() {
		pgina.adicionarComponente(fabrica.criarComponente(boto, 12, 13));
		pgina.adicionarComponente(fabrica.criarComponente(etiqueta, 21, 2));

		final String endereo = System.getProperty("user.home")
				+ System.getProperty("file.separator") + "paginaTeste.ged";

		serializador.salvarPginaEmArquivo(pgina, endereo);

		final TipoPaginaGedix pginaRecuperada = serializador.recuperarPgina(endereo);

		assertEquals(pgina, pginaRecuperada);
	}
}
