package testes.infraestrutura;

import static infraestrutura.TiposDeComponentesGedix.boto;
import static infraestrutura.TiposDeComponentesGedix.caixaDeEscolha;
import static org.junit.Assert.assertEquals;
import infraestrutura.excecoes.ExcecaoDeArquivo;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import infraestrutura.serializacao.SerializadorXML;
import infraestrutura.serializacao.TipoSerializadorXML;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import org.junit.Before;
import org.junit.Test;
import pagina.PaginaGedix;
import pagina.TipoPaginaGedix;


public class TestesDoSerializadorXML {

	private TipoSerializadorXML serializador;
	private TipoFabrica fabrica;
	private TipoPaginaGedix pgina;

	@Test(expected = ExcecaoDeArquivo.class)
	public void aoSeTentarSalvarUmaPginaXMLNumCaminhoInexistenteUmaExceoDeArquivoLanada() {
		serializador.salvarArquivoXML(pgina, "/aflalihrhrr");
	}

	@Test(expected = ExcecaoDeArquivo.class)
	public void aoSeTentarSalvarUmaPginaXMLSemPermissoLanadaUmaExceoDeArquivo() {
		if (System.getProperty("os.name").toLowerCase().contains("linux"))
			if (!System.getProperty("user.name").equals("root"))
				serializador.salvarArquivoXML(pgina, "/usr/bin/pagina1.dix.xml");
	}

	private String lerStringDoArquivo(String caminho) throws IOException {
		final File arquivo = new File(caminho);
		final FileInputStream stream = new FileInputStream(arquivo);
		final byte[] bytes = new byte[(int) arquivo.length()];

		stream.read(bytes);
		arquivo.delete();
		return new String(bytes);
	}

	@Test
	public void oSerializadorXMLPodeGerarUmaPginaDixXMLAPartirDeUmaPginaGedix()
			throws IOException {
		pgina.adicionarComponente(fabrica.criarComponente(boto, 30, 40));
		pgina.adicionarComponente(fabrica.criarComponente(caixaDeEscolha, 11, 0));

		final String endereo = System.getProperty("user.home")
				+ System.getProperty("file.separator") + "paginaQualquer.dix.xml";

		serializador.salvarArquivoXML(pgina, endereo);

		assertEquals(pgina.gerarXML(), lerStringDoArquivo(endereo));
	}

	@Before
	public void prepararTestes() {
		serializador = new SerializadorXML();
		fabrica = FabricaDeComponentesGedix.instncia();
		pgina = new PaginaGedix("nomeDaPagina", "ttulo");
	}
}
