package testes.pagina;

import static infraestrutura.TiposDeComponentesGedix.boto;
import static infraestrutura.TiposDeComponentesGedix.grade;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import org.junit.Before;
import org.junit.Test;
import pagina.PaginaGedix;
import pagina.TipoPaginaGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;


public class TestesDeIgualdadeEntrePaginas {

	private TipoPaginaGedix pagina;
	private TipoFabrica fabricaDoGedix;
	private TipoComponenteGedix b;
	private TipoComponenteGedix g;

	@Before
	public void prepararTestes() {
		final String nomeDaPagina = "nomeQualquer";
		final String ttuloDaPagina = "titulo";

		pagina = new PaginaGedix(nomeDaPagina, ttuloDaPagina);
		fabricaDoGedix = FabricaDeComponentesGedix.instncia();
		b = fabricaDoGedix.criarComponente(boto, 15, 10);
		g = fabricaDoGedix.criarComponente(grade, 34, 12);

		pagina.adicionarComponente(b);
		pagina.adicionarComponente(g);
	}

	@Test
	public void umaPginaComOMesmoNomeEComponentesDiferentesDeOutraDiferenteDessaOutra() {
		final TipoComponenteGedix b2 = fabricaDoGedix.criarComponente(boto, 15, 12);
		final TipoComponenteGedix g2 = fabricaDoGedix.criarComponente(grade, 30, 12);

		final TipoPaginaGedix pginaDiferente = new PaginaGedix("nomeQualquer", "titulo");
		pginaDiferente.adicionarComponente(b2);
		pginaDiferente.adicionarComponente(g2);

		assertFalse(pginaDiferente.equals(pagina));
	}

	@Test
	public void umaPginaComOsMesmosComponentesDeUmaOutraIgualEssaOutra() {
		final TipoPaginaGedix pginaIgual = new PaginaGedix("outra", "titulo");
		pginaIgual.adicionarComponente(b);
		pginaIgual.adicionarComponente(g);

		assertEquals(pagina, pginaIgual);
	}

}
