package testes.pagina;

import static infraestrutura.TiposDeComponentesGedix.boto;
import static infraestrutura.TiposDeComponentesGedix.caixaDeEscolha;
import static infraestrutura.TiposDeComponentesGedix.grade;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import infraestrutura.fabrica.FabricaDeComponentesGedix;
import infraestrutura.fabrica.TipoFabrica;
import org.junit.Before;
import org.junit.Test;
import pagina.PaginaGedix;
import pagina.TipoPaginaGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;


public class TestesDePagina {

	private TipoPaginaGedix pagina;
	private TipoFabrica fabricaDoGedix;

	@Test
	public void aPginaPodeAdicionarUmProxyDeComponente() {
		final TipoComponenteGedix proxyDeGrade = fabricaDoGedix.criarComponente(grade, 0,
				0);
		pagina.adicionarComponente(proxyDeGrade);

		assertTrue(pagina.obterComponentes().contains(proxyDeGrade));
	}

	@Test
	public void aPginaPodeFornecerUmProxyDeComponenteSendoDadoSeuNome() {
		final TipoComponenteGedix proxyDeCaixaDeEscolha = fabricaDoGedix.criarComponente(
				caixaDeEscolha, 10, 10);
		pagina.adicionarComponente(proxyDeCaixaDeEscolha);

		final String nomeDaCaixa = proxyDeCaixaDeEscolha.obterNome();

		assertEquals(proxyDeCaixaDeEscolha, pagina.obterComponente(nomeDaCaixa));
	}

	@Test
	public void aPginaPodeRemoverUmProxyDeComponenteSendoDadoSeuNome() {
		final TipoComponenteGedix proxyDeBoto = fabricaDoGedix.criarComponente(boto, 1,
				1);
		pagina.adicionarComponente(proxyDeBoto);

		final String nomeDoBoto = proxyDeBoto.obterNome();

		pagina.removerComponente(nomeDoBoto);

		assertFalse(pagina.obterComponentes().contains(proxyDeBoto));
	}

	@Test
	public void oXMLDaPginaContmOXMLDeSeusComponentes() {
		final TipoComponenteGedix caixaDeEscolha1 = fabricaDoGedix.criarComponente(
				caixaDeEscolha, 10, 10);

		final TipoComponenteGedix boto1 = fabricaDoGedix
				.criarComponente(boto, 100, 100);

		pagina.adicionarComponente(caixaDeEscolha1);
		pagina.adicionarComponente(boto1);

		final String xmlDaCaixa = caixaDeEscolha1.gerarXML();
		final String xmlDoBoto = boto1.gerarXML();

		assertTrue(pagina.gerarXML().contains(xmlDaCaixa));
		assertTrue(pagina.gerarXML().contains(xmlDoBoto));
	}

	@Test
	public void oXMLDaPginaContmSeuNomeETtulo() {
		final String xmlEsperado = "<pgina nome=\"nomeQualquer\" ttulo=\"titulo\"";
		assertTrue(pagina.gerarXML().contains(xmlEsperado));
	}

	@Test
	public void oXMLDaPginaSempreComeaCom2LinhasPadro() {
		final String primeiraLinha = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
		final String segundaLinha = "<?xml-stylesheet type=\"text/xsl\" href=\"/dix/dix_2007-10-23/transformacao/dix.xml\"?>";

		assertTrue(pagina.gerarXML().contains(primeiraLinha));
		assertTrue(pagina.gerarXML().contains(segundaLinha));
	}

	@Test
	public void podeSeObterONomeDeUmaPgina() {
		assertEquals("nomeQualquer", pagina.obterNome());
	}

	@Before
	public void prepararTestes() {
		final String nomeDaPagina = "nomeQualquer";
		final String ttuloDaPagina = "titulo";

		pagina = new PaginaGedix(nomeDaPagina, ttuloDaPagina);

		fabricaDoGedix = FabricaDeComponentesGedix.instncia();
	}

	@Test
	public void umaPginaContmOToStringDeSeusComponentesNoSeuToString() {
		String reprEsperada = "";

		pagina.adicionarComponente(fabricaDoGedix.criarComponente(boto, 120, 9));

		for (final TipoComponenteGedix c : pagina.obterComponentes())
			reprEsperada = reprEsperada + c.toString() + "\n";

		assertEquals(reprEsperada, pagina.toString());
	}

}
