package testes.proxies;

import static org.junit.Assert.assertEquals;
import infraestrutura.excecoes.DimensoesDeGradeInvalidas;
import infraestrutura.excecoes.ExcecaoGedix;
import org.junit.Before;
import org.junit.Test;
import proxies.ProxyDeGrade;
import testes.proxies.abstratos.TestesDeProxyDeComponenteGedix;
import componentes.abstratos.gedix.TipoComponenteGedix;
import componentes.grade.Grade;
import componentes.grade.TipoGrade;


public class TestesDeProxyDeGrade extends TestesDeProxyDeComponenteGedix {

	private TipoGrade grade;
	private TipoGrade proxyDeGrade;

	protected TipoComponenteGedix componente() {
		return grade;
	}

	@Before
	public void prepararTestes() {
		grade = new Grade();
		proxyDeGrade = new ProxyDeGrade(grade);
		proxy = proxyDeGrade;
	}

	@Test(expected = DimensoesDeGradeInvalidas.class)
	public void seMandarUmNmeroDeColunasInvlidoOProxyNoAtualizaAGrade()
			throws ExcecaoGedix {
		final int numInvlidoDeColunas = 0;

		grade.fixarLinhas(10);
		proxyDeGrade.fixarLinhas(numInvlidoDeColunas);

		assertEquals(10, grade.obterLinhas());
	}

	@Test
	public void seMandarUmNmeroDeColunasVlidoOProxyAtualizaAGrade() throws ExcecaoGedix {
		proxyDeGrade.fixarColunas(10);
		assertEquals(10, grade.obterColunas());
	}

	@Test(expected = DimensoesDeGradeInvalidas.class)
	public void seMandarUmNmeroDeLinhasInvlidoOProxyNoAtualizaAGrade()
			throws ExcecaoGedix {
		final int numInvlidoDeLinhas = 0;

		grade.fixarLinhas(15);
		proxyDeGrade.fixarLinhas(numInvlidoDeLinhas);

		assertEquals(15, grade.obterLinhas());
	}

	@Test
	public void seMandarUmNmeroDeLinhasVlidoOProxyAtualizaAGrade() throws ExcecaoGedix {
		proxyDeGrade.fixarLinhas(6);
		assertEquals(6, grade.obterLinhas());
	}

}