package testes.proxies.abstratos;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import infraestrutura.PosicoesDeLegenda;
import infraestrutura.excecoes.DimensaoInvalida;
import infraestrutura.excecoes.PosicaoInvalida;
import org.junit.Test;
import componentes.abstratos.gedix.TipoComponenteGedix;


public abstract class TestesDeProxyDeComponenteGedix {

	protected TipoComponenteGedix proxy;

	protected abstract TipoComponenteGedix componente();

	@Test
	public void seMandarOAtualizadorAtualizarAAlturaDeUmComponenteEleOFazSeForAlturaVlida() {
		final int novaAltura = 20;

		proxy.fixarAltura(novaAltura);

		assertEquals(novaAltura, componente().obterAltura());
	}

	@Test
	public void seMandarOAtualizadorAtualizarAEsquerdaDeUmComponenteEleOFazSeForEsquerdaVlida() {
		final int novaEsquerda = 45;

		proxy.fixarEsquerda(novaEsquerda);

		assertEquals(novaEsquerda, componente().obterEsquerda());
	}

	@Test
	public void seMandarOAtualizadorAtualizarALarguraDeUmComponenteEleOFazSeForLarguraVlida() {
		final int novaLargura = 120;

		proxy.fixarLargura(novaLargura);

		assertEquals(novaLargura, componente().obterLargura());
	}

	@Test
	public void seMandarOAtualizadorAtualizarOTopoDeUmComponenteEleOFazSeOTopoForVlido() {
		final int novoTopo = 20;

		proxy.fixarTopo(novoTopo);

		assertEquals(novoTopo, componente().obterTopo());
	}

	@Test
	public void seMandarOAtualizadorDesabilitarUmComponenteEleOFaz() {
		proxy.desabilitar();
		assertFalse(componente().estHabilitado());
	}

	@Test
	public void seMandarOAtualizadorFixarALegendaDeUmComponenteEleOFaz() {
		final String novaLegenda = "legendaQualquer";
		proxy.fixarLegenda(novaLegenda);

		assertEquals(novaLegenda, componente().obterLegenda());
	}

	@Test
	public void seMandarOAtualizadorFixarAPosioDaLegendaDeUmComponenteEleOFaz() {
		final PosicoesDeLegenda novaPosio = PosicoesDeLegenda.emCima;
		proxy.fixarPosioDeLegenda(novaPosio);

		assertEquals(novaPosio, componente().obterPosioDeLegenda());
	}

	@Test
	public void seMandarOAtualizadorFixarProfundidadeDeUmComponenteEleOFaz() {
		final int novaProfundidade = 10;
		proxy.fixarProfundidade(novaProfundidade);

		assertEquals(10, componente().obterProfundidade());
	}

	@Test
	public void seMandarOAtualizadorHabilitarUmComponenteEleOFaz() {
		componente().desabilitar();

		proxy.habilitar();
		assertTrue(componente().estHabilitado());
	}

	@Test
	public void seMandarOAtualizadorTornarUmComponenteInvisvelEleOFaz() {
		componente().tornarVisvel();
		proxy.tornarInvisvel();

		assertFalse(componente().estVisvel());
	}

	@Test
	public void seMandarOAtualizadorTornarUmComponenteVisvelEleOFaz() {
		componente().tornarInvisvel();
		proxy.tornarVisvel();

		assertTrue(componente().estVisvel());
	}

	@Test(expected = DimensaoInvalida.class)
	public void seMandarUmValorDeAlturaInvlidoEleNoAtualizaOComponente() {
		final int alturaInvlida = 0;

		componente().fixarAltura(15);
		proxy.fixarAltura(alturaInvlida);

		assertEquals(15, componente().obterAltura());
	}

	@Test(expected = PosicaoInvalida.class)
	public void seMandarUmValorDeEsquerdaInvlidoEleNoAtualizaOComponente() {
		final int esquerdaInvlida = -32;

		componente().fixarEsquerda(105);
		proxy.fixarEsquerda(esquerdaInvlida);

		assertEquals(105, componente().obterEsquerda());
	}

	@Test(expected = DimensaoInvalida.class)
	public void seMandarUmValorDeLarguaInvlidoEleNoAtualizaOComponente() {
		final int larguraInvlida = -10;

		componente().fixarLargura(15);
		proxy.fixarLargura(larguraInvlida);

		assertEquals(15, componente().obterLargura());
	}

	@Test(expected = PosicaoInvalida.class)
	public void seMandarUmValorDeTopoInvlidoEleNoAtualizaOComponente() {
		final int topoInvlido = -210;

		componente().fixarTopo(15);
		proxy.fixarTopo(topoInvlido);

		assertEquals(15, componente().obterTopo());
	}

	@Test
	public void umProxyDeComponenteGedixTemToStringIgualAoDoComponentePorEleCuidado() {
		assertEquals(componente().toString(), proxy.toString());
	}

}
